/*************************************************************************
 *
 *  Demonstrate grab from two asynchronous sources in real time on the
 *  DAC module. The grabs are double-buffered offscreen then copied to
 *  the visible portion of the frame buffer.
 *
 *  KEY_COLOR, DCF0 and DCF1 defines must be set properly.
 *
 */

#include <imapi.h>
#include <process.h>
#include <time.h>

#define KEY_COLOR 8
#define DCF0 "rs170"
#define DCF1 "ccir"

typedef struct
{
   long thr[2];
   long cam;
   long dig;
   long osb[2];
   long control;
   long copyControl;
   long offScreen[2];
   long disp;
   long proc;
   long channel;
   long display;
   long count;
} GRABINFO;

GRABINFO info[2];

void error(void *param);
unsigned int __stdcall grab(void *param);
long stop[2];
long copy = 1, dbuf = 1;


/*---------------------------------------------------------------------------
 *
 * Name:          main()
 *
 * Synopsis:      Main body.
 *
 * Returns:       none
 *
 */

#if defined(_IM_COMP_MICROSOFT)
void main(int argc, char *argv[])
{
   long dev, thr, dispMode, key = 1, index, regen = 0, color = 0, dsync = 0;
   long xsize[2], ysize[2], winStop[2], one = 1, two = 1, chan, dispXsize;
   unsigned int addrOne, addrTwo;
   double startTime, endTime;
   HANDLE handleOne, handleTwo;

   memset(&info, 0, sizeof(info));

   if (argc > 1 && *argv[1] == '?')
   {
      printf("\nUsage: dacdbuf [options]\n");
      printf("               -nokey      Turn keying off\n");
      printf("               -disable n  Disable the desired channel: 0 or 1\n");
      printf("               -nodouble   Disable double-buffering\n");
      printf("               -color      Do not set the display in monochome\n");
      printf("               -nocopy     Disable copies from offscreen to visible display\n");
      printf("               -dsync      Enable synchronization to display for channel 1\n");
      printf("               -regen      Reprogram channel before every grab\n");
      exit(1);
   }
   else
      printf("Type 'dacdbuf ?' for help\n");

   printf("Current DCF are:\nCh0 - '%s'  Ch1 - '%s'\n", DCF0, DCF1);
   for (index = 1; index < argc; index++)
   {
      if (!strcmp(argv[index], "-nokey"))
         key = 0;
      else if (!strcmp(argv[index], "-color"))
         color = 1;
      else if (!strcmp(argv[index], "-nocopy"))
         copy = 0;
      else if (!strcmp(argv[index], "-nodouble"))
         dbuf = 0;
      else if (!strcmp(argv[index], "-dsync"))
         dsync = 1;
      else if (!strcmp(argv[index], "-disable"))
      {
         long dis;

         sscanf(argv[index + 1], "%li", &dis);

         if (dis)
            two = 0;
         else if (!dis)
            one = 0;
      }
      else if (!strcmp(argv[index], "-regen"))
         regen = 1;
   }

   printf("Loading base board...\n");
   imAppCatchError(IM_DEFAULT, error, NULL);
   imDevAlloc(0, 0, NULL, IM_DEFAULT, &dev);
   imThrAlloc(dev, 0, &thr);

   /* Clear display */
   imBufChild(thr, IM_DISP, 0, 0, IM_ALL, IM_ALL, &info[0].display);
   imBufClear(thr, info[0].display, 0, 0);
   imBufInquire(thr, info[0].display, IM_BUF_SIZE_X, &dispXsize);

   winStop[0] = (dispXsize >> 1) - 1;
   winStop[1] = (dispXsize >> 1) - 1;

   /* Adjust keying */
   if (key && ((imDispInquire(thr, 0, IM_DISP_VGA_MODE, &dispMode) == IM_SINGLE_SCREEN) || (dispMode == IM_DUAL_HEAD)))
   {
      imBufPutField(thr, info[0].display, IM_DISP_KEY_MODE, IM_KEY_IN_RANGE);
      imBufPutField(thr, info[0].display, IM_DISP_KEY_LOW, KEY_COLOR);
      imBufPutField(thr, info[0].display, IM_DISP_KEY_HIGH, KEY_COLOR);
      if (!color) imBufPutField(thr, info[0].display, IM_DISP_MODE, IM_DISP_MONO);
      imDispControl(thr, 0, info[0].display, IM_NOW);
   }

   for (chan = 0; chan < 2; chan++)
   {
      info[chan].channel = chan;
      imThrAlloc(dev, 0, &info[chan].thr[0]);
      imThrAlloc(dev, 0, &info[chan].thr[1]);
      imDigAlloc(thr, 0, chan, IM_DEFAULT, &info[chan].dig);
      imSyncAlloc(thr, &info[chan].osb[0]);
      imSyncAlloc(thr, &info[chan].osb[1]);
      imBufAllocControl(thr, &info[chan].control);
      imBufAllocControl(thr, &info[chan].copyControl);
      imCamAlloc(thr, chan ? DCF1 : DCF0, IM_DEFAULT, &info[chan].cam);

      imCamInquire(thr, info[chan].cam, IM_DIG_SIZE_X, &xsize[chan]);
      imCamInquire(thr, info[chan].cam, IM_DIG_SIZE_Y, &ysize[chan]);
      imBufPutField(thr, info[chan].control, IM_CTL_STOP_X, xsize[chan] < winStop[chan] ? xsize[chan] - 1 : winStop[chan] );
      if (dsync)
         imBufPutField(thr, info[chan].copyControl, IM_CTL_DISPLAY_SYNC, chan ? IM_ENABLE : IM_DISABLE);

      imBufChild(thr, chan ? IM_DISP_RED : IM_DISP_GREEN,                    0, 0, winStop[chan] + 1, ysize[chan], &info[chan].offScreen[0]);
      imBufChild(thr, chan ? IM_DISP_RED : IM_DISP_GREEN,          winStop[chan] + 1, 0, winStop[chan] + 1, ysize[chan], &info[chan].offScreen[1]);
      imBufChild(thr,                       IM_DISP_BLUE, !chan ? 0 : winStop[0] + 1, 0, winStop[chan] + 1, ysize[chan], &info[chan].disp);

      if (!regen)
         imBufPutField(thr, info[chan].control, IM_CTL_REGEN_LIST, IM_DISABLE);
   }

   imBufClear(thr, info[0].display, 0, 0);
   imSyncHost(thr, 0, IM_COMPLETED);

   stop[0] = stop[1] = 0;
   startTime = imSysClock(thr, 0.0);

   /* Start both channels */
   if (one)
      handleOne = (HANDLE) _beginthreadex(NULL, 0, grab, (void *)&info[0].channel, 0, &addrOne);
   else
      stop[0] = 1;

   if (two)
      handleTwo = (HANDLE) _beginthreadex(NULL, 0, grab, (void *)&info[1].channel, 0, &addrTwo);
   else
      stop[1] = 1;

   while (!stop[0] || !stop[1])
      Sleep(100);

   endTime = imSysClock(thr, startTime);
   printf("\nch0: %2.1f f/s, ch1: %2.1f f/s\n", info[0].count / endTime, info[1].count / endTime);


   /* Cleanup */
   for (chan = 0; chan < 2; chan++)
   {
      imBufFree(thr, info[chan].osb[0]);
      imBufFree(thr, info[chan].osb[1]);
      imBufFree(thr, info[chan].control);
      imBufFree(thr, info[chan].copyControl);
      imBufFree(thr, info[0].display);
      imDigFree(thr, info[chan].dig);
      imCamFree(thr, info[chan].cam);
   }
   imThrFree(thr);
   imDevFree(dev);

}


/*---------------------------------------------------------------------------
 *
 * Name:          grab()
 *
 * Synopsis:      Double-buffered loop.
 *
 * Designed:      rBec for MACAB
 *                8/28/98   5:57:25 PM
 *
 * Returns:
 *
 */

unsigned int __stdcall grab(void *param)
{
   GRABINFO *locInfo = &info[*(long *)param];
   long buf;

   /* Loop of double-buffered grabs */
   if (dbuf)
      imDigGrab(locInfo->thr[0], locInfo->dig, locInfo->cam, locInfo->offScreen[0] , 1, locInfo->control, locInfo->osb[0]);

   buf = 1;
   while (!kbhit())
   {
      locInfo->count++;

      /* Grab into an area of the frame buffer that is not visible */
      imDigGrab(locInfo->thr[0], locInfo->dig, locInfo->cam, locInfo->offScreen[buf], 1, locInfo->control, locInfo->osb[buf]);

      /* Switch buffers */
      if (dbuf)
         buf = 1 - buf;

      /* Wait until the previous grab has completed */
      imSyncHost(locInfo->thr[1], locInfo->osb[buf], IM_COMPLETED);

      /* Optional processing or copy to visible portion of frame buffer,
       * while the next frame is being grabbed. When this is over, the
       * next grab command is sent so that the board can be programmed
       * in time for the next frame to be grabbed.
       */
      if (copy)
         imBufCopy(locInfo->thr[1], locInfo->offScreen[buf], locInfo->disp, locInfo->copyControl, 0);

   }

   stop[locInfo->channel] = 1;

   return(0);
}



/*---------------------------------------------------------------------------
 *
 * Name:          error()
 *
 * Synopsis:      Error handler.
 *
 * Returns:       none
 *
 */

void error(void *param)
{
    char Error[IM_ERR_SIZE];

    imAppGetError(IM_ERR_MSG_FUNC, Error);
    printf("\n%s\n", Error);

    exit(0);
}

#endif
