/*************************************************************************
 *
 *  Demonstrate grab from two asynchronous sources on the DAC module.
 *  Grab is done directly to display.
 *
 *  KEY_COLOR, DCF0 and DCF1 defines must be set properly.
 *
 */

#include <imapi.h>
#include <time.h>

#define KEY_COLOR 8

#define DCF0 "rs170"
#define DCF1 "ccir"

void error(void *Param);


/*---------------------------------------------------------------------------
 *
 * Name:          main()
 *
 * Synopsis:      Main body.
 *
 * Returns:       none
 *
 */

void main(int argc, char *argv[])
{
   long display;
   long one = 1, two = 1;
   long dev[1];
   long thr[3];
   long osb[2], disp[2], digitizer[2], cam[2], control[2];
   long winStop[2], xsize[2], ysize[2];
   long dispMode, dispXsize, dispYsize, key = 1, index, iter;
   char dcf[2][256] = { DCF0, DCF1 };

   if (argc > 1 && *argv[1] == '?')
   {
      printf("\nUsage: dacgrab [options]\n");
      printf("               -nokey      Turn keying off\n");
      printf("               -disable n  Disable the desired channel: 0 or 1\n");
      exit(1);
   }
   else
      printf("Type 'dacgrab ?' for help\n");

   for (index = 1; index < argc; index++)
   {
      if (!strcmp(argv[index], "-nokey"))
         key = 0;
      else if (!strcmp(argv[index], "-disable"))
      {
         long dis;

         sscanf(argv[index + 1], "%li", &dis);

         if (dis)
            two = 0;
         else if (!dis)
            one = 0;
      }
   }

   printf("Loading base board...\n");
   imAppCatchError(IM_DEFAULT, error, NULL);
   imDevAlloc(0, 0, NULL, IM_DEFAULT, &dev[0]);
   imThrAlloc(dev[0], 0, &thr[0]);
   imThrAlloc(dev[0], 0, &thr[1]);
   imThrAlloc(dev[0], 0, &thr[2]);

   for (index = 0; index < 2; index++)
   {
      imDigAlloc(thr[0], 0, index, IM_DEFAULT, &digitizer[index]);
      imSyncAlloc(thr[0], &osb[index]);
      imBufAllocControl(thr[0], &control[index]);
      imCamAlloc(thr[0], dcf[index], IM_DEFAULT, &cam[index]);
      imCamInquire(thr[0], cam[index], IM_DIG_SIZE_X, &xsize[index]);
      imCamInquire(thr[0], cam[index], IM_DIG_SIZE_Y, &ysize[index]);
   }

   /* Adjust keying */
   if (key && ((imDispInquire(thr[0], 0, IM_DISP_VGA_MODE, &dispMode) == IM_SINGLE_SCREEN) || (dispMode == IM_DUAL_HEAD)))
   {
      imBufPutField(thr[0], control[0], IM_DISP_KEY_MODE, IM_KEY_IN_RANGE);
      imBufPutField(thr[0], control[0], IM_DISP_KEY_LOW, KEY_COLOR);
      imBufPutField(thr[0], control[0], IM_DISP_KEY_HIGH, KEY_COLOR);
      imBufPutField(thr[0], control[0], IM_DISP_MODE, IM_DISP_MONO);
      imDispControl(thr[0], 0, control[0], IM_NOW);
   }


   /* Clear display */
   imBufChild(thr[0], IM_DISP, 0, 0, IM_ALL, IM_ALL, &display);
   imBufClear(thr[0], display, 0, 0);
   imBufInquire(thr[0], display, IM_BUF_SIZE_X, &dispXsize);
   imBufInquire(thr[0], display, IM_BUF_SIZE_Y, &dispYsize);

   winStop[0] = xsize[0] - 1;
   winStop[1] = (xsize[1] < (dispXsize-xsize[0])) ? xsize[1] - 1 : dispXsize - xsize[0] - 1;

   if (two)
      imBufPutField(thr[0], control[0], IM_CTL_STOP_X, winStop[0]);
   if (one)
      imBufPutField(thr[0], control[1], IM_CTL_STOP_X, winStop[1]);

   if (one)
   {
      if (two)
         imBufChild(thr[0], IM_DISP, 0, 0, winStop[0] + 1, ysize[0], &disp[0]);
      else
         imBufChild(thr[0], IM_DISP, 0, 0, xsize[0], ysize[0], &disp[0]);
   }

   if (two)
   {
      if (one)
         imBufChild(thr[0], IM_DISP, winStop[0] + 1, 0, winStop[1] + 1, ysize[1], &disp[1]);
      else
         imBufChild(thr[0], IM_DISP, 0, 0, xsize[1], ysize[1], &disp[1]);
   }

   imBufClear(thr[0], display, 0, 0);
   imSyncHost(thr[0], 0, IM_COMPLETED);

   iter = 0;
   printf("Grab from two asynchronous sources\n");
   while (!kbhit())
   {
      printf("%d\r", iter++);

      /* Start a grab in each channel, simultaneously */
      if (one) imDigGrab(thr[0], digitizer[0], cam[0], disp[0], 1, control[0], osb[0]);
      if (two) imDigGrab(thr[1], digitizer[1], cam[1], disp[1], 1, control[1], osb[1]);

      /* Wait for completion */
      if (one) imSyncHost(thr[2], osb[0], IM_COMPLETED);
      if (two) imSyncHost(thr[2], osb[1], IM_COMPLETED);

   }

   if (one) imBufFree(thr[0], disp[0]);
   if (two) imBufFree(thr[1], disp[1]);

   imBufFree(thr[0], control[0]);
   imBufFree(thr[1], control[1]);
}


/*---------------------------------------------------------------------------
 *
 * Name:          error()
 *
 * Synopsis:      Error handler.
 *
 * Returns:       none
 *
 */

void error(void *Param)
{
    char Error[IM_ERR_SIZE];
    
    /* Just to prevent warnings */
    Param = Param;

    imAppGetError(IM_ERR_MSG_FUNC, Error);
    printf("\n%s\n", Error);

    exit(0);
}
