// MDisplayOverlay example

// Synopsis:   This program shows how to display an image in an ActiveMIL 
//             display while doing text and graphics annotations 
//             on top of it using ActiveMIL GraphicContext control and 
//             Windows GDI drawing. If the target system supports grab, 
//             the annotations are done on top of live video.
// 
//             Note that if true VGA overlay (color keying) is supported 
//             by the target system, the annotations are done in the VGA 
//             overlay surface, otherwhise, overlay is simulated by MIL 
//             and pseudo-live grab will be used if grabbing.

// NOTE: Check the history logs to determine which properties were set a 
//       design time.

// ExampleDlg.cpp: implementation file
//

#include "stdafx.h"
#include "ExampleDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


// Location of image file
const char EXAMPLE_IMAGE_PATH[] = "..\\..\\..\\..\\..\\images\\board.mim";  


BEGIN_MESSAGE_MAP(CExampleDlg, CDialog)
	//{{AFX_MSG_MAP(CExampleDlg)
	ON_BN_CLICKED(IDC_NEXTBUTTON, OnNextbutton)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// The one and only CExampleApp object
//
CExampleApp theApp;


/////////////////////////////////////////////////////////////////////////////
// CExampleDlg dialog
//
CExampleDlg::CExampleDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CExampleDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CExampleDlg)
	m_strCommentText = _T("");
	//}}AFX_DATA_INIT

   m_nStepCount = 1;
}


/////////////////////////////////////////////////////////////////////////////
// CExampleDlg data exchange
//
void CExampleDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CExampleDlg)
	DDX_Text(pDX, IDC_COMMENTTEXT, m_strCommentText);
	DDX_Control(pDX, IDC_IMAGE_BUFFER, m_ImageBuffer);
	DDX_Control(pDX, IDC_DISPLAY, m_Display);
	DDX_Control(pDX, IDC_SYSTEM, m_System);
	DDX_Control(pDX, IDC_DIGITIZER, m_Digitizer);
	DDX_Control(pDX, IDC_LEFT_OVERLAY_GRAPHIC_CONTEXT, m_LeftOverlayGraphicContext);
	DDX_Control(pDX, IDC_RIGHT_OVERLAY_GRAPHIC_CONTEXT, m_RightOverlayGraphicContext);
	//}}AFX_DATA_MAP
}


/////////////////////////////////////////////////////////////////////////////
// CExampleDlg message handlers
//
BOOL CExampleDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

   ///////
   // Step 1 : grab continuously

   // If the system can grab
   if (m_System.GetNumberOfDigitizers())
      {
      // allocate the digitizer
      m_Digitizer.Allocate();

      // Adjust image buffer to the size of the digitizer
      m_ImageBuffer.SetSizeX(m_Digitizer.GetSizeX());
      m_ImageBuffer.SetSizeY(m_Digitizer.GetSizeY());
      m_ImageBuffer.SetCanGrab(TRUE);
      m_ImageBuffer.Allocate();

      }
   // If the system cannot grab
   else
      {
      MessageBox("No digitizer present, "
                 "an example image will be loaded into the image.\n\n"
                 "Click OK to continue.  ");

      // Load example image
      m_ImageBuffer.Load(EXAMPLE_IMAGE_PATH, TRUE);
      }

	// Resize the dialog 
   AdjustDialog();

   // If a digitizer was allocated, start continuous grab
   if (m_Digitizer.GetIsAllocated())
      {
      m_Digitizer.GrabContinuous();
      }

   m_strCommentText = "This program shows how to do "
                      "overlay annotations in an ActiveMIL display.  "
                      "Click Next to continue.  ";

	UpdateData(FALSE);
	return TRUE;  
}


/////////////////////////////////////////////////////////////////////////////
// OnNextButton message handler.
// This function drives the example through each step, performing appropriate
// actions
//
void CExampleDlg::OnNextbutton() 
{
   // Increments the step counter and updates text in the dialog 
   AdjustStepNumber();

   switch (m_nStepCount) 
      {
      ///////
      // Step 2: Draw overlay annotations
      case 2:
         {
         // Draw overlay annotations
         OverlayDraw();

         m_strCommentText = "Overlay annotations has been added to image.  "
                            "Click Next to continue.  ";
         break;
         }

      ///////
      // Step 3: Stop the grab and display the last image
      case 3:
         {
         if (m_Digitizer.GetIsAllocated())
            {
            m_Digitizer.Halt();
            
            m_strCommentText = "Displaying the last grabbed image with overlay annotations.  "
                               "Click Next to end.  ";
            }
         else
            {
            // For the non-grabbing case, close dialog application now
            EndDialog(IDOK);
            }
         break;
         }

      ///////
      // Last Step: End the application
      default:
         {
         // Close dialog application
         EndDialog(IDOK);
         break;
         }
      }

   UpdateData(FALSE);
}


/////////////////////////////////////////////////////////////////////////////
// OverlayDraw.
// This function enables and writes graphics into the overlay image
//
void CExampleDlg::OverlayDraw()
{
   IRegionOfInterest DrawingRegion;

   // Use a custom Look Up Table to perform color annotation when the
   // overlay is single band (typically 256 colors)
   if (m_Display.GetOverlayImage().GetNumberOfBands() == 1)
      {
      // Create a grey scale Look Up Table
      m_Display.GetOverlayLUT().GenerateRamp(0, 0, 255, 255);

      // Add the desired annotation colors in it
      m_Display.GetOverlayLUT().GenerateColor(1, 1, m_LeftOverlayGraphicContext.GetForegroundColor());
      m_Display.GetOverlayLUT().GenerateColor(2, 2, m_RightOverlayGraphicContext.GetForegroundColor());  
      m_Display.GetOverlayLUT().GenerateColor(3, 3, RGB(255, 0, 0));
      m_Display.GetOverlayLUT().GenerateColor(4, 4, RGB(255, 255, 0)); 

      m_Display.SetOverlayLUTEnabled(TRUE);
      }

   // Clear the overlay image, this makes the overlay transparent
   m_Display.GetOverlayImage().Clear(m_Display.GetOverlayKeyColor());

   // Enable overlay display 
   m_Display.SetOverlayVisible(TRUE);

   ///////
   // Draw monochrome overlay annotation
   //
   // Get Overlay size x and y
   long ImageWidth, ImageHeight;
   ImageWidth  = m_ImageBuffer.GetSizeX();
   ImageHeight = m_ImageBuffer.GetSizeY();

   // Print a string in the overlay image buffer
   DrawingRegion = m_LeftOverlayGraphicContext.GetDrawingRegion();
   DrawingRegion.SetMode(mCenterSize);
   DrawingRegion.SetSizeX(140);
   DrawingRegion.SetSizeY(20);
   DrawingRegion.SetCenterX(ImageWidth / 4);
   DrawingRegion.SetCenterY(ImageHeight / 4);
   m_LeftOverlayGraphicContext.Text("ActiveMIL Overlay");

   // Draw a box around the string
   DrawingRegion.SetCenterX(ImageWidth / 4 - 2);
   m_LeftOverlayGraphicContext.Rectangle(FALSE);

   ///////
   // Draw color overlay annotation 
   //
   // Print a string in the overlay image buffer
   DrawingRegion = m_RightOverlayGraphicContext.GetDrawingRegion();
   DrawingRegion.SetMode(mCenterSize);
   DrawingRegion.SetSizeX(140);
   DrawingRegion.SetSizeY(20);
   DrawingRegion.SetCenterX(ImageWidth * 3 / 4);
   DrawingRegion.SetCenterY(ImageHeight / 4);
   m_RightOverlayGraphicContext.Text("ActiveMIL Overlay");

   // Draw a box around the string
   DrawingRegion.SetCenterX(ImageWidth * 3 / 4 - 2);
   m_RightOverlayGraphicContext.Rectangle(FALSE);


   ///////
   // Draw GDI color overlay annotation - available only the display type is not in 
   //                                     dual-screen mode or non windowed mode
   //
   if (m_Display.GetDisplayType() != mDualScreen && m_Display.GetDisplayType() != mNonWindowed)
   {
      IImage OverlayImage = m_Display.GetOverlayImage();

      // Create a device context to draw in the overlay image with GDI. 
      OverlayImage.Control(mWindowDCAlloc, COleVariant((long)mDefault));

      // Get the device context
      CDC dc; 
      HDC hdc = (HDC) OverlayImage.Inquire(mWindowDC);
      dc.Attach(hdc);

      // Create a blue pen
      CPen pen(PS_SOLID, 1, RGB(0, 0, 255)); 
      CPen *pOldPen = dc.SelectObject(&pen);
      	
      // Draw a cross in the overlay image
      dc.MoveTo(0,            ImageHeight/2);
      dc.LineTo(ImageWidth,   ImageHeight/2);
      dc.MoveTo(ImageWidth/2, 0);
      dc.LineTo(ImageWidth/2, ImageHeight);
      
      // Write text in the overlay image
      dc.SetBkMode(TRANSPARENT);
      dc.SetTextColor(RGB(255, 0, 0));
      dc.TextOut(ImageWidth*3/18, ImageHeight*4/6, "GDI Overlay Text ");
      dc.SetTextColor(RGB(255, 255, 0));
      dc.TextOut(ImageWidth*12/18, ImageHeight*4/6, "GDI Overlay Text ");
      
      // Select back the old pen
      dc.SelectObject(pOldPen);

      // Delete created device context
      dc.Detach();
      OverlayImage.Control(mWindowDCFree, COleVariant((long)mDefault));
   
      // Tell ActiveMIL that the overlay image was modified
      OverlayImage.Modified(mContents);
   }
}

