// MMultiThreadLite example

// Synopsis:   This program shows how to use different threads and synchronize 
//             them with MIL. It creates 4 processing threads that are used
//             to work in 4 different regions of a display Image.
//
//     Thread usage:
//      - The main thread starts a processing thread in each of the 4 different
//        quarters of a display Image. The main thread then waits for a the user 
//        to click on the Next button.
//      - The top-left and bottom-left threads work in a loop, as follows: the
//        top-left thread draws a rectangle in its Image, then sends an event to
//        the bottom-left thread. The bottom-left thread waits for the event
//        from the top-left thread, copies the top-left Image in the bottom-left Image,
//        draws a circle in it and then sends an event to the top-left thread.
//        When the top-left thread receives the event, the loop continues.
//      - The top-right and bottom-right threads work exactly the same way as the
//        top-left and bottom-left threads.
//
//      Note that the top and bottom threads (of each half) could be set to do
//      something else while waiting for each other. 

// NOTE: Check the history logs to determine which properties were set at
//       design time.

// ExampleDlg.cpp: implementation file
//

#include "stdafx.h"
#include "ExampleDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


// Constants for processing
const long DRAW_RADIUS_MAX  = 50;
                                 

// Custom Windows message 
const int WM_PROCESSINGWORKFUNCTIONEND = WM_USER + 1;


// Location of image file
const char EXAMPLE_IMAGE_PATH[] = "..\\..\\..\\..\\..\\images\\bird.mim";  



BEGIN_MESSAGE_MAP(CExampleDlg, CDialog)
	//{{AFX_MSG_MAP(CExampleDlg)
	ON_BN_CLICKED(IDC_NEXTBUTTON, OnNextbutton)
	ON_MESSAGE(WM_PROCESSINGWORKFUNCTIONEND, OnProcessingWorkFunctionsEnd)
	ON_WM_CLOSE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// The one and only CExampleApp object
//
CExampleApp theApp;


/////////////////////////////////////////////////////////////////////////////
// CExampleDlg dialog
//
CExampleDlg::CExampleDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CExampleDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CExampleDlg)
	m_strCommentText = _T("");
	//}}AFX_DATA_INIT

   // intialize members variables
   m_nStepCount            = 1;
   m_NumberOfTopRight      = 0;
   m_NumberOfTopLeft       = 0;
   m_NumberOfBottomLeft    = 0; 
   m_NumberOfBottomRight   = 0;
   m_ComVarLeft            = 0;
   m_ComVarRight           = 0;
   m_bKillAllThreads       = FALSE;
   m_bThreadsStillRunning  = FALSE;
}


/////////////////////////////////////////////////////////////////////////////
// CExampleDlg data exchange
//
void CExampleDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CExampleDlg)
	DDX_Control(pDX, IDC_NEXTBUTTON, m_NextButton);
	DDX_Text(pDX, IDC_COMMENTTEXT, m_strCommentText);
	DDX_Control(pDX, IDC_DISPLAY, m_Display);
	DDX_Control(pDX, IDC_IMAGE_BUFFER, m_ImageBuffer);
	DDX_Control(pDX, IDC_SYSTEM, m_System);
	DDX_Control(pDX, IDC_APPLICATION, m_Application);
	DDX_Control(pDX, IDC_BOTTOM_LEFT_CHILD_IMAGE, m_BottomLeftChildImage);
	DDX_Control(pDX, IDC_BOTTOM_LEFT_DRAW_GRAPHICCONTEXT, m_BottomLeftDrawGraphicContext);
	DDX_Control(pDX, IDC_BOTTOM_LEFT_GRAPHICCONTEXT, m_BottomLeftGraphicContext);
	DDX_Control(pDX, IDC_BOTTOM_RIGHT_CHILD_IMAGE, m_BottomRightChildImage);
	DDX_Control(pDX, IDC_BOTTOM_RIGHT_DRAW_GRAPHICCONTEXT, m_BottomRightDrawGraphicContext);
	DDX_Control(pDX, IDC_BOTTOM_RIGHT_GRAPHICCONTEXT, m_BottomRightGraphicContext);
	DDX_Control(pDX, IDC_TOP_LEFT_DRAW_GRAPHICCONTEXT, m_TopLeftDrawGraphicContext);
	DDX_Control(pDX, IDC_TOP_LEFT_GRAPHICCONTEXT, m_TopLeftGraphicContext);
	DDX_Control(pDX, IDC_TOP_RIGHT_CHILD_IMAGE, m_TopRightChildImage);
	DDX_Control(pDX, IDC_TOP_RIGHT_DRAW_GRAPHICCONTEXT, m_TopRightDrawGraphicContext);
	DDX_Control(pDX, IDC_TOP_RIGHT_GRAPHICCONTEXT, m_TopRightGraphicContext);
	DDX_Control(pDX, IDC_TOP_LEFT_CHILD_IMAGE, m_TopLeftChildImage);
	//}}AFX_DATA_MAP
}


/////////////////////////////////////////////////////////////////////////////
// CExampleDlg message handlers
//
BOOL CExampleDlg::OnInitDialog()
{
	CDialog::OnInitDialog();


	// Resize the dialog 
   AdjustDialog(EXAMPLE_IMAGE_PATH);

   ///////
   // Step 1 : Start the various worker threads 

   // Allocate ActiveMIL synchronization events
   m_EventSendTopLeft    = m_Application.ControlThread(mDefault, mEventAlloc, mDefault);
   m_EventSendTopRight   = m_Application.ControlThread(mDefault, mEventAlloc, mDefault);
   m_EventWaitTopLeft    = m_Application.ControlThread(mDefault, mEventAlloc, mDefault);
   m_EventWaitTopRight   = m_Application.ControlThread(mDefault, mEventAlloc, mDefault);
   m_EventEndTopLeft     = m_Application.ControlThread(mDefault, mEventAlloc, mDefault);
   m_EventEndBottomLeft  = m_Application.ControlThread(mDefault, mEventAlloc, mDefault);
   m_EventEndTopRight    = m_Application.ControlThread(mDefault, mEventAlloc, mDefault);
   m_EventEndBottomRight = m_Application.ControlThread(mDefault, mEventAlloc, mDefault);

   // Initialize source images
   m_TopLeftChildImage.Load(EXAMPLE_IMAGE_PATH, FALSE);
   m_TopRightChildImage.Load(EXAMPLE_IMAGE_PATH, FALSE);
   m_BottomLeftChildImage.Clear(0);   
   m_BottomRightChildImage.Clear(0);

   // Start threads
   AfxBeginThread(TopLeftThreadFunction,     this);
   AfxBeginThread(TopRightThreadFunction,    this);
   AfxBeginThread(BottomLeftThreadFunction,  this);
   AfxBeginThread(BottomRightThreadFunction, this);

   // Print message
   m_strCommentText = "This program demonstrates the use of multiple threads and their "
                      "synchronization using ActiveMIL thread events.  "
                      "Click Next to continue.  ";
   
	UpdateData(FALSE);
	return TRUE;  
}


/////////////////////////////////////////////////////////////////////////////
// OnNextButton message handler.
// This function drives the example through each step, performing appropriate
// actions
//
void CExampleDlg::OnNextbutton() 
{
   // Increments the step counter and updates text in the dialog 
   AdjustStepNumber();

   switch (m_nStepCount)
      {
      ///////
      // Step 2: Trigger start of thread control functions
      case 2:
         {
         // Send events to trigger start of operation of top left and top right threads
         m_Application.ControlThread(m_EventWaitTopLeft,  mEventSet, mSignaled);
         m_Application.ControlThread(m_EventWaitTopRight, mEventSet, mSignaled);

         m_bThreadsStillRunning = TRUE;

         m_strCommentText = "Processing done in loops.  "
                            "Click Next to end.  ";
         break;
         }


      ///////
      // Step 3: Trigger stop of thread control functions 
      case 3:
         {
         // Send events to trigger stop of operation of top left and top right threads
         m_Application.ControlThread(m_EventEndTopLeft , mEventSet, mSignaled);
         m_Application.ControlThread(m_EventEndTopRight, mEventSet, mSignaled);

         // Disable Next button and print waiting message
         m_NextButton.EnableWindow(FALSE);
         m_strCommentText = "Waiting for threads to complete...  ";
         break;
         }

   
      ///////
      // Last Step: end the application
      default:
         {
         // Close dialog application
         EndDialog(IDOK);
         break;
         }
      }
	UpdateData(FALSE);
}


/////////////////////////////////////////////////////////////////////////////
// OnProcessingWorkFunctionEnd message handler
// This function handles WM_TOPPROCESSINGWORKFUNCTIONEND custom Windows message
// posted by the processing thread control function at end of processing
//
LONG CExampleDlg::OnProcessingWorkFunctionsEnd(UINT, LONG)
{
   static long nCount = 0;

   // ...sequel of Step 3: Trigger stop of thread control functions 
   // wait for the fourth message to be received, meaning all threads 
   // have terminated
   if (++nCount >= 4)
      {
      // Reenable Next button
      m_NextButton.EnableWindow(TRUE);
      m_NextButton.SetFocus();

      // Print message
      CString str;
      str.Format("Top left iterations done: %4ld.  "
                 "Bottom left iterations done: %4ld.  "
                 "Top right iterations done: %4ld.  "
                 "Bottom right iterations done: %4ld.  "
                 "Click Next to end.  ",
                 m_NumberOfTopLeft,
                 m_NumberOfBottomLeft,
                 m_NumberOfTopRight,
                 m_NumberOfBottomRight);

      m_strCommentText = str;
      UpdateData(FALSE);

      // Free ActiveMIL synchronization events
      m_Application.ControlThread(m_EventSendTopLeft,    mEventFree, mDefault);
      m_Application.ControlThread(m_EventSendTopRight,   mEventFree, mDefault);
      m_Application.ControlThread(m_EventWaitTopLeft,    mEventFree, mDefault);
      m_Application.ControlThread(m_EventWaitTopRight,   mEventFree, mDefault);
      m_Application.ControlThread(m_EventEndTopLeft,     mEventFree, mDefault);
      m_Application.ControlThread(m_EventEndBottomLeft,  mEventFree, mDefault);
      m_Application.ControlThread(m_EventEndTopRight,    mEventFree, mDefault);
      m_Application.ControlThread(m_EventEndBottomRight, mEventFree, mDefault);

      m_bThreadsStillRunning = FALSE;

      // If the user has closed the window, end now.
      if (m_bKillAllThreads)
         EndDialog(IDOK);
      }
   return 0;
}


/////////////////////////////////////////////////////////////////////////////
// TopLeftWorkFunction
// This function performs the real work of the TopLeft processing thread
//
void CExampleDlg::TopLeftWorkFunction()
{
   BOOL Exit = FALSE;

   while (! Exit)
      {
      // Wait for event to process
      m_Application.ControlThread(m_EventWaitTopLeft, mEventWait, mDefault);

      // Modify communication variable and reload image if necessary
      m_ComVarLeft += 10;   
      if (m_ComVarLeft > DRAW_RADIUS_MAX)
         {
         m_ComVarLeft = 10;   
         m_TopLeftChildImage.Load(EXAMPLE_IMAGE_PATH, FALSE);
         }
      
      // Draw rectangle
      IRegionOfInterest DrawingRegion = m_TopLeftDrawGraphicContext.GetDrawingRegion();
      DrawingRegion.SetSizeX(2 * m_ComVarLeft);
      DrawingRegion.SetSizeY(2 * m_ComVarLeft);
      m_TopLeftDrawGraphicContext.Rectangle(TRUE);

      // Increment iteration count and draw text
      m_NumberOfTopLeft += 1;   
      CString str;
      str.Format("%d", m_NumberOfTopLeft);
      m_TopLeftGraphicContext.Text(str);
      
      // Check if processing must be terminated
      if ((m_Application.ControlThread(m_EventEndTopLeft, mEventState, mDefault) == mSignaled)
          || m_bKillAllThreads)
         {
         // Make bottom thread exit
         m_Application.ControlThread(m_EventEndBottomLeft, mEventSet, mSignaled);
         
         // Set exit loop flag
         Exit = TRUE;
         }

      // Synchronize bottom thread with end of processing
      m_Application.ControlThread(m_EventSendTopLeft, mEventSet, mSignaled);

      // Minimize impact of Window's thread scheduling adjustment mechanisms. 
      BalanceThreadScheduling();
      }
}


/////////////////////////////////////////////////////////////////////////////
// BottomLeftWorkFunction
// This function performs the real work of the BottomLeft processing thread
//
void CExampleDlg::BottomLeftWorkFunction()
{
   BOOL Exit = FALSE;

   while (! Exit)
      {
      // Wait for event to process
      m_Application.ControlThread(m_EventSendTopLeft, mEventWait, mDefault);

      // Copy top image
      m_BottomLeftChildImage.Copy(m_TopLeftChildImage, mAllBands);
      
      // Draw cirle
      IRegionOfInterest DrawingRegion = m_BottomLeftDrawGraphicContext.GetDrawingRegion();
      DrawingRegion.SetSizeX(2 * m_ComVarLeft);
      DrawingRegion.SetSizeY(2 * m_ComVarLeft);
      m_BottomLeftDrawGraphicContext.Arc(0.0, 360.0, TRUE);
   
      // Increment iteration count and draw text
      m_NumberOfBottomLeft += 1; 
      CString str;
      str.Format("%d", m_NumberOfBottomLeft);
      m_BottomLeftGraphicContext.Text(str);
   
      // Check if processing must be terminated
      if ((m_Application.ControlThread(m_EventEndBottomLeft, mEventState, mDefault) == mSignaled)
          || m_bKillAllThreads)
         {
         // Set exit loop flag
         Exit = TRUE;
         }

      // Synchronize top thread with end of processing
      m_Application.ControlThread(m_EventWaitTopLeft, mEventSet, mSignaled);

      // Minimize impact of Window's thread scheduling adjustment mechanisms. 
      BalanceThreadScheduling();
      }
}


/////////////////////////////////////////////////////////////////////////////
// TopRightWorkFunction
// This function performs the real work of the TopRight processing thread
//
void CExampleDlg::TopRightWorkFunction()
{
   BOOL Exit = FALSE;

   while (! Exit)
      {
      // Wait for event to process
      m_Application.ControlThread(m_EventWaitTopRight, mEventWait, mDefault);
      
      // Modify communication variable and reload image if necessary
      m_ComVarRight += 10;   
      if (m_ComVarRight > DRAW_RADIUS_MAX)
         {
         m_ComVarRight = 10;   
         m_TopRightChildImage.Load(EXAMPLE_IMAGE_PATH, FALSE);
         }
      
      // Draw rectangle
      IRegionOfInterest DrawingRegion = m_TopRightDrawGraphicContext.GetDrawingRegion();
      DrawingRegion.SetSizeX(2 * m_ComVarRight);
      DrawingRegion.SetSizeY(2 * m_ComVarRight);
      m_TopRightDrawGraphicContext.Rectangle(TRUE);

      // Increment iteration count and draw text
      m_NumberOfTopRight += 1;   
      CString str;
      str.Format("%d", m_NumberOfTopRight);
      m_TopRightGraphicContext.Text(str);
   
      // Modify communication variable
      m_ComVarRight += 10;   
      
      // Check if processing must be terminated
      if ((m_Application.ControlThread(m_EventEndTopRight, mEventState, mDefault) == mSignaled)
          || m_bKillAllThreads)
         {
         // Make bottom thread exit
         m_Application.ControlThread(m_EventEndBottomRight, mEventSet, mSignaled);
         
         // Set exit loop flag
         Exit = TRUE;
         }
      // Synchronize bottom thread with end of processing
      m_Application.ControlThread(m_EventSendTopRight, mEventSet, mSignaled);

      // Minimize impact of Window's thread scheduling adjustment mechanisms. 
      BalanceThreadScheduling();
      }
}



/////////////////////////////////////////////////////////////////////////////
// BottomRightWorkFunction
// This function performs the real work of the BottomRight processing thread
//
void CExampleDlg::BottomRightWorkFunction()
{
   BOOL Exit = FALSE;

   while (! Exit)
      {
      // Wait for event to process
      m_Application.ControlThread(m_EventSendTopRight, mEventWait, mDefault);

      // Copy top image
      m_BottomRightChildImage.Copy(m_TopRightChildImage, mAllBands);
      
      // Draw cirle
      IRegionOfInterest DrawingRegion = m_BottomRightDrawGraphicContext.GetDrawingRegion();
      DrawingRegion.SetSizeX(m_ComVarRight);
      DrawingRegion.SetSizeY(m_ComVarRight);
      m_BottomRightDrawGraphicContext.Arc(0.0, 360.0, TRUE);
   
      // Increment iteration count and draw text
      m_NumberOfBottomRight+= 1;   

      CString str;
      str.Format("%d", m_NumberOfBottomRight);
      m_BottomRightGraphicContext.Text(str);
   
      // Check if processing must be terminated
      if ((m_Application.ControlThread(m_EventEndBottomRight, mEventState, mDefault) == mSignaled)
          || m_bKillAllThreads)
         {
         // Set exit loop flag
         Exit = TRUE;
         }
      // Synchronize top thread with end of processing
      m_Application.ControlThread(m_EventWaitTopRight, mEventSet, mSignaled);

      // Minimize impact of Window's thread scheduling adjustment mechanisms. 
      BalanceThreadScheduling();
      }
}


/////////////////////////////////////////////////////////////////////////////
// TopLeftThreadFunction
// This function controls the TopLeft processing thread
//
UINT CExampleDlg::TopLeftThreadFunction(LPVOID pParam)
{
   CExampleDlg *pthis = (CExampleDlg *) pParam;

   // Call TopLeft processing loop
   //
   // Enclose ActiveMIL calls in a try-catch statement in order
   // to prevent ActiveMIL errors to cause an uncaught 
   // exception message.  MFC does not catch exceptions thrown by
   // ActiveX controls outside of the message loop.
   try
      {
      pthis->TopLeftWorkFunction();
      }
   catch (COleDispatchException *e)
      {
      // Display error message
      e->ReportError();

      // Always call Delete() on caught exceptions
      e->Delete();

      // Kill application
      ExitProcess(1);
      }


   // Post a custom Windows message to inform the main UI thread of
   // the processing completion.
   //
   // A custom message handler will then be triggered and 
   // perform the remaining tasks of the current example step.
   //
   // This is the preferred way of communicating with worker threads in
   // MFC. An alternative way would be to use synchronized wait functions
   // such as WaitForSingleObject(MyThreadHandle) to wait for thread 
   // termination. But this way, UI components would be frozen and the 
   // application would stop responding. 
   //
   // Hence, we recommend that all lengthy processing be performed in a work 
   // thread in such a way as not to block the main UI thread.
   pthis->PostMessage(WM_PROCESSINGWORKFUNCTIONEND);
   return 0;
}


/////////////////////////////////////////////////////////////////////////////
// BottomLeftThreadFunction
// This function controls the BottomLeft processing thread
//
UINT CExampleDlg::BottomLeftThreadFunction(LPVOID pParam)
{
   CExampleDlg *pthis = (CExampleDlg *) pParam;

   // Call BottomLeftprocessing loop
   //
   // Enclose ActiveMIL calls in a try-catch statement in order
   // to prevent ActiveMIL errors to cause an uncaught 
   // exception message.  MFC does not catch exceptions thrown by
   // ActiveX controls outside of the message loop.
   try
      {
      pthis->BottomLeftWorkFunction();
      }
   catch (COleDispatchException *e)
      {
      // Display error message
      e->ReportError();

      // Always call Delete() on caught exceptions
      e->Delete();

      // Kill application
      ExitProcess(1);
      }

   // Post a custom Windows message to inform the main UI thread of
   // the processing completion.
   //
   // A custom message handler will then be triggered and 
   // perform the remaining tasks of the current example step.
   //
   // This is the preferred way of communicating with worker threads in
   // MFC. An alternative way would be to use synchronized wait functions
   // such as WaitForSingleObject(MyThreadHandle) to wait for thread 
   // termination. But this way, UI components would be frozen and the 
   // application would stop responding. 
   //
   // Hence, we recommend that all lengthy processing be performed in a work 
   // thread in such a way as not to block the main UI thread.
   pthis->PostMessage(WM_PROCESSINGWORKFUNCTIONEND);
   return 0;
}


/////////////////////////////////////////////////////////////////////////////
// TopLeftThreadFunction
// This function controls the TopRight processing thread
//
UINT CExampleDlg::TopRightThreadFunction(LPVOID pParam)
{
   CExampleDlg *pthis = (CExampleDlg *) pParam;

   // Call TopRight processing loop
   //
   // Enclose ActiveMIL calls in a try-catch statement in order
   // to prevent ActiveMIL errors to cause an uncaught 
   // exception message.  MFC does not catch exceptions thrown by
   // ActiveX controls outside of the message loop.
   try
      {
      pthis->TopRightWorkFunction();
      }
   catch (COleDispatchException *e)
      {
      // Display error message
      e->ReportError();

      // Always call Delete() on caught exceptions
      e->Delete();

      // Kill application
      ExitProcess(1);
      }

   // Post a custom Windows message to inform the main UI thread of
   // the processing completion.
   //
   // A custom message handler will then be triggered and 
   // perform the remaining tasks of the current example step.
   //
   // This is the preferred way of communicating with worker threads in
   // MFC. An alternative way would be to use synchronized wait functions
   // such as WaitForSingleObject(MyThreadHandle) to wait for thread 
   // termination. But this way, UI components would be frozen and the 
   // application would stop responding. 
   //
   // Hence, we recommend that all lengthy processing be performed in a work 
   // thread in such a way as not to block the main UI thread.
   pthis->PostMessage(WM_PROCESSINGWORKFUNCTIONEND);
   return 0;
}


/////////////////////////////////////////////////////////////////////////////
// BottomRightThreadFunction
// This function controls the BottomRight processing thread
//
UINT CExampleDlg::BottomRightThreadFunction(LPVOID pParam)
{
   CExampleDlg *pthis = (CExampleDlg *) pParam;

   // Call BottomRight processing loop
   //
   // Enclose ActiveMIL calls in a try-catch statement in order
   // to prevent ActiveMIL errors to cause an uncaught 
   // exception message.  MFC does not catch exceptions thrown by
   // ActiveX controls outside of the message loop.
   try
      {
      pthis->BottomRightWorkFunction();
      }
   catch (COleDispatchException *e)
      {
      // Display error message
      e->ReportError();

      // Always call Delete() on caught exceptions
      e->Delete();

      // Kill application
      ExitProcess(1);
      }

   // Post a custom Windows message to inform the main UI thread of
   // the processing completion.
   //
   // A custom message handler will then be triggered and 
   // perform the remaining tasks of the current example step.
   //
   // This is the preferred way of communicating with worker threads in
   // MFC. An alternative way would be to use synchronized wait functions
   // such as WaitForSingleObject(MyThreadHandle) to wait for thread 
   // termination. But this way, UI components would be frozen and the 
   // application would stop responding. 
   //
   // Hence, we recommend that all lengthy processing be performed in a work 
   // thread in such a way as not to block the main UI thread.
   pthis->PostMessage(WM_PROCESSINGWORKFUNCTIONEND);
   return 0;
}

void CExampleDlg::OnClose() 
{
   if (m_bThreadsStillRunning)
      {
      // Instruct all threads to terminate
      m_bKillAllThreads = TRUE;

      // Don't call the base class, all the threads have to be finished 
      // before DestroyWindow is called. See OnProcessingWorkFunctionsEnd.
      }
   else
      CDialog::OnClose();
}
