// MMultiThreadLite example

// Synopsis:  This program shows how to use different threads and synchronize 
//            them with MIL. It creates 4 processing threads that are used
//            to work in 4 different regions of a display buffer.
//
//     Thread usage:
//      - The main thread starts a processing thread in each of the 4 different
//        quarters of a display buffer. The main thread then waits for a the user 
//        to click on the Next button.
//      - The top-left and bottom-left threads work in a loop, as follows: the
//        top-left thread draws a rectangle in its buffer, then sends an event to
//        the bottom-left thread. The bottom-left thread waits for the event
//        from the top-left thread, copies the top-left buffer in the bottom-left buffer,
//        draws a circle in it and then sends an event to the top-left thread.
//        When the top-left thread receives the event, the loop continues.
//      - The top-right and bottom-right threads work exactly the same way as the
//        top-left and bottom-left threads.
//
//      Note that the top and bottom threads (of each half) could be set to do
//      something else while waiting for each other. 

// ExampleDlg.h : header file
//
//{{AFX_INCLUDES()

#include "Display.h"
#include "System.h"
#include "Application.h"
#include "GraphicContext.h"
#include "Image.h"
//}}AFX_INCLUDES

#if !defined(AFX_EXAMPLEDLG_H_MMultiThreadLite_INCLUDED_)
#define AFX_EXAMPLEDLG_H_MMultiThreadLite_INCLUDED_

#include "resource.h"		// main symbols

/////////////////////////////////////////////////////////////////////////////
// CExampleDlg dialog
//
class CExampleDlg : public CDialog
{
// Construction
public:
	CExampleDlg(CWnd* pParent = NULL);	// standard constructor

// Dialog Data
	//{{AFX_DATA(CExampleDlg)
	enum { IDD = IDD_EXAMPLE_DIALOG };
	CButton           m_NextButton;
	CString           m_strCommentText;
	CDisplay          m_Display;
	CImage            m_ImageBuffer;
	CSystem           m_System;
	CApplication      m_Application;
	CImage            m_BottomLeftChildImage;
	CGraphicContext   m_BottomLeftDrawGraphicContext;
	CGraphicContext   m_BottomLeftGraphicContext;
	CImage            m_BottomRightChildImage;
	CGraphicContext   m_BottomRightDrawGraphicContext;
	CGraphicContext   m_BottomRightGraphicContext;
	CGraphicContext   m_TopLeftDrawGraphicContext;
	CGraphicContext   m_TopLeftGraphicContext;
	CImage            m_TopRightChildImage;
	CGraphicContext   m_TopRightDrawGraphicContext;
	CGraphicContext   m_TopRightGraphicContext;
	CImage            m_TopLeftChildImage;
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CExampleDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);	// DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
   long m_nStepCount;            // Keeps track of the current example step 
   void AdjustDialog(CString strImagePath);  // Resizes the dialog to fit any font size and digitizer resolution
   void AdjustStepNumber();      // Increments the step number and updates text in the dialog


   long m_NumberOfTopLeft;          // Number of top right threads iterations
   long m_NumberOfTopRight;         // Number of top right threads iterations
   long m_NumberOfBottomLeft;       // Number of bottom left threads iterations
   long m_NumberOfBottomRight;      // Number of bottom right threads iterations
   long m_ComVarLeft;               // Communication variable for left thread
   long m_ComVarRight;              // Communication variable for right thread

   long m_EventSendTopLeft;         // Event send by top left thread
   long m_EventSendTopRight;        // Event send by top right thread
   long m_EventWaitTopLeft;         // Event waited on by top left thread
   long m_EventWaitTopRight;        // Event waited on by top right thread
   long m_EventEndTopLeft;          // Event used to exit top left thread
   long m_EventEndBottomLeft;       // Event used to exit bottom left thread
   long m_EventEndTopRight;         // Event used to exit top right thread
   long m_EventEndBottomRight;      // Event used to exit bottom right thread

   bool m_bKillAllThreads;          // Flag for threads termination
   bool m_bThreadsStillRunning;     // Flag to indicate there are threads still running

   // Thread Scheduling Adjustment Handling 
   // ------------------------------------- 
   // NOTE: In some situations, Windows adjusts the priority or quantum value of threads.   
   //       The intent of these adjustments is to improve system throughput and             
   //       responsiveness. However, like any scheduling algorithms these adjustment are    
   //       not perfect, and they might not be beneficial to all applications. The          
   //       following function is introduced to minimize the impact of thread scheduling    
   //       adjustment as performed by Windows. Refer to the book "Inside Windows NT        
   //       (Second Edition, Microsoft Press, 1998, ISBN 1-57231-677-2)" for more details   
   //       about thread scheduling under Windows.                                          
   void BalanceThreadScheduling()
   {
      Sleep(0);
   }
   
   // processing threads control functions
   static UINT TopLeftThreadFunction   (LPVOID pParam);
   static UINT TopRightThreadFunction  (LPVOID pParam);
   static UINT BottomLeftThreadFunction(LPVOID pParam);
   static UINT BottomRightThreadFunction(LPVOID pParam);

   // processing threads working functions
   void TopLeftWorkFunction();
   void TopRightWorkFunction();
   void BottomLeftWorkFunction();
   void BottomRightWorkFunction();

   // Custom Windows message handler
   afx_msg LONG OnProcessingWorkFunctionsEnd( UINT, LONG );

	// Generated message map functions
	//{{AFX_MSG(CExampleDlg)
	virtual BOOL OnInitDialog();
	afx_msg void OnNextbutton();
	afx_msg void OnClose();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};


/////////////////////////////////////////////////////////////////////////////
// CExampleApp:
// See Example.cpp for the implementation of this class
//
class CExampleApp : public CWinApp
{
public:
   CExampleApp() {};

	//{{AFX_VIRTUAL(CExampleApp)
	inline virtual LRESULT ProcessWndProcException( CException* e, const MSG* pMsg );
	inline virtual BOOL InitInstance();
	//}}AFX_VIRTUAL

};


/////////////////////////////////////////////////////////////////////////////
// CExampleApp initialization
//
BOOL CExampleApp::InitInstance()
{
	AfxEnableControlContainer();

	CExampleDlg dlg;
	m_pMainWnd = &dlg;
	dlg.DoModal();

	return FALSE;
}


/////////////////////////////////////////////////////////////////////////////
// CExampleApp default exception handling
 
LRESULT CExampleApp::ProcessWndProcException( CException* e, const MSG* pMsg )
{
   e->ReportError() ;
   static_cast<CDialog*>(m_pMainWnd)->EndDialog(IDCANCEL) ;
 
   return 1 ;
}
 
 
//{{AFX_INSERT_LOCATION}}
// Microsoft Developer Studio will insert additional declarations immediately before the previous line.

#endif 
