// MDoubleBufferingLite example

// Synopsis:   This example does double buffered grab with real time 
//             processing. It also times to see if some frames are not 
//             processed using hooks.    
//
//             Note this assumes that the processing operation is shorter 
//             than a grab and that the PC has sufficient bandwith 
//             to support the 2 operations simultaneously. Also if the 
//             target processing image is not on the display, the 
//             processing speed is augmented.       

// NOTE: Check the history logs to determine which properties were set at 
//       design time.

// ExampleDlg.cpp: implementation file
//

#include "stdafx.h"
#include "ExampleDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


// Custom Windows message 
const int WM_PROCESSINGWORKFUNCTIONEND = WM_USER + 1;


BEGIN_MESSAGE_MAP(CExampleDlg, CDialog)
	//{{AFX_MSG_MAP(CExampleDlg)
	ON_BN_CLICKED(IDC_NEXTBUTTON, OnNextbutton)
	ON_MESSAGE(WM_PROCESSINGWORKFUNCTIONEND, OnProcessingWorkFunctionEnd)
	ON_WM_CLOSE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// The one and only CExampleApp object
//
CExampleApp theApp;


/////////////////////////////////////////////////////////////////////////////
// CExampleDlg dialog
//
CExampleDlg::CExampleDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CExampleDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CExampleDlg)
	m_strCommentText = _T("");
	//}}AFX_DATA_INIT

   m_nStepCount = 1;
   m_nProcessingCount = 0;
}


/////////////////////////////////////////////////////////////////////////////
// CExampleDlg data exchange
//
void CExampleDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CExampleDlg)
	DDX_Control(pDX, IDC_NEXTBUTTON, m_NextButton);
	DDX_Text(pDX, IDC_COMMENTTEXT, m_strCommentText);
	DDX_Control(pDX, IDC_IMAGE_BUFFER, m_ImageBuffer);
	DDX_Control(pDX, IDC_DISPLAY, m_Display);
	DDX_Control(pDX, IDC_DIGITIZER, m_Digitizer);
	DDX_Control(pDX, IDC_SYSTEM, m_System);
	DDX_Control(pDX, IDC_APPLICATION, m_Application);
	DDX_Control(pDX, IDC_GRAB_IMAGE_1, m_GrabImage1);
	DDX_Control(pDX, IDC_GRAB_IMAGE_2, m_GrabImage2);
	DDX_Control(pDX, IDC_GRAPHIC_CONTEXT_1, m_GraphicContext1);
	DDX_Control(pDX, IDC_GRAPHIC_CONTEXT_2, m_GraphicContext2);
	//}}AFX_DATA_MAP
}


/////////////////////////////////////////////////////////////////////////////
// CExampleDlg message handlers
//
BOOL CExampleDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Resize the dialog 
   AdjustDialog();

   ///////
   // Step 1: Resize and allocate the Image Buffer, and start the grab.

   m_ImageBuffer.SetSizeX((long) (m_Digitizer.GetSizeX() * m_Digitizer.GetScaleX()));
   m_ImageBuffer.SetSizeY((long) (m_Digitizer.GetSizeY() * m_Digitizer.GetScaleY()));
   m_ImageBuffer.Allocate();
   m_Digitizer.GrabContinuous();
   m_strCommentText = "This program demonstrates double buffered processing.  "
                      "Click Next to continue.  ";

	UpdateData(FALSE);
	return TRUE;  
}


/////////////////////////////////////////////////////////////////////////////
// OnNextButton message handler.
// This function drives the example through each step, performing appropriate
// actions
//
void CExampleDlg::OnNextbutton() 
{
   // Increments the step counter and updates text in the dialog 
   AdjustStepNumber();

   switch (m_nStepCount)
      {
      ///////
      // Step 2: Grab the sequence
      case 2:
         {
         // Stop continuous grab 
         m_Digitizer.Halt();
         
         // Start processing thread
         m_bStopProcessing = FALSE;
         m_bKillProcessing = FALSE;
         AfxBeginThread(ProcessingThreadFunction, this);

         //Print message
         m_strCommentText = "Double buffered processing (annotate and copy) in progress... "
                            "Click Next to continue. ";
         break;
         }

      ///////
      // Step 3: Print statistics
      case 3:
         {
         // Instruct the processing thread to stop its loop
         m_bStopProcessing = TRUE;

         // Disable Next button and print waiting message
         m_NextButton.EnableWindow(FALSE);
         m_strCommentText = "Waiting for processing to complete...  ";

         // To be continued in OnProcessingWorkFunctionEnd...
         break;
         }

      ///////
      // Last Step: End the application
      default:
         {
         // Close dialog application
         EndDialog(IDOK);
         break;
         }
      }
	UpdateData(FALSE);
}


/////////////////////////////////////////////////////////////////////////////
// ProcessingThreadFunction
// This function controls the processing thread
//
UINT CExampleDlg::ProcessingThreadFunction(LPVOID pParam)
{
   CExampleDlg *pthis = (CExampleDlg *) pParam;

   // Call CExampleDlg's processing loop
   //
   // Enclose ActiveMIL calls in a try-catch statement in order
   // to prevent ActiveMIL errors to cause an uncaught 
   // exception message.  MFC does not catch exceptions thrown by
   // ActiveX controls outside of the message loop.
   try
      {
      pthis->ProcessingWorkFunction();
      }
   catch (COleDispatchException *e)
      {
      // Display error message
      e->ReportError();

      // Always call Delete() on caught exceptions
      e->Delete();

      // Kill application
      ExitProcess(1);
      }

   // Post a custom Windows message to inform the main UI thread of
   // the processing completion.
   //
   // A custom message handler will then be triggered and 
   // perform the remaining tasks of the current example step.
   //
   // This is the preferred way of communicating with worker threads in
   // MFC. An alternative way would be to use synchronised wait functions
   // such as WaitForSingleObject(MyThreadHandle) to wait for thread 
   // termination. But this way, UI components would be frozen and the 
   // application would stop responding. 
   //
   // Hence, we recommend that all lengthy processing be performed in a work 
   // thread in such a way as not to block the main UI thread.
   pthis->PostMessage(WM_PROCESSINGWORKFUNCTIONEND);
   
   return 0;
}


/////////////////////////////////////////////////////////////////////////////
// ProcessingWorkFunction
// This function performs the real work of the processing thread
//
void CExampleDlg::ProcessingWorkFunction()
{

   // Adjust sizes of grab images to that of effective digitizer size
   //and allocate them
   m_GrabImage1.SetSizeX(m_ImageBuffer.GetSizeX());
   m_GrabImage1.SetSizeY(m_ImageBuffer.GetSizeY());
   m_GrabImage1.Allocate();

   m_GrabImage2.SetSizeX(m_ImageBuffer.GetSizeX());
   m_GrabImage2.SetSizeY(m_ImageBuffer.GetSizeY());
   m_GrabImage2.Allocate();

   // Grab the first image
   m_Digitizer.SetImage(m_GrabImage1);
   m_Digitizer.Grab();

   // Start the timer
   m_Application.GetTimer().Reset();

   // Process one image while grabbing the other
   CString str;
   while (! m_bStopProcessing && ! m_bKillProcessing)
      {
      // Grab second image while processing first image
      m_Digitizer.SetImage(m_GrabImage2);
      m_Digitizer.Grab();

      // Synchronize and start the timer
      if (m_nProcessingCount == 0)
         m_Application.GetTimer().Reset();

      // Process first image already grabbed
      // NOTE: Real time only if PC is fast enough
      str.Format("%d",m_nProcessingCount + 1);
      m_GraphicContext1.Text(str);
      m_ImageBuffer.Copy(m_GrabImage1, mAllBands);

      // Count processed images
      m_nProcessingCount++;

      // Grab first image while processing second image
      m_Digitizer.SetImage(m_GrabImage1);
      m_Digitizer.Grab();

      // Process second image already grabbed
      // NOTE: Real time only if PC is fast enough
      str.Format("%d",m_nProcessingCount + 1);
      m_GraphicContext2.Text(str);
      m_ImageBuffer.Copy(m_GrabImage2, mAllBands);

      // Count processed images
      m_nProcessingCount++;
      }

   if (! m_bKillProcessing)
      {

      // Wait for the last grab end and stop the timer.
      m_Digitizer.GrabWait(mGrabEnd);

      // Get the time elapsed in processing loop
      m_dTime = m_Application.GetTimer().Read();
   
      // Get the time elapsed in processing loop
      m_dTime = m_Application.GetTimer().Read();

      }
}


/////////////////////////////////////////////////////////////////////////////
// OnProcessingWorkFunctionEnd message handler
// This function handles WM_PROCESSINGWORKFUNCTIONEND custom Windows message
// posted by the processing thread control function at end of processing
//
LONG CExampleDlg::OnProcessingWorkFunctionEnd(UINT, LONG)
{
   // ...sequel of Step 3: Print statistics
   if (! m_bKillProcessing)
      {

      // Reenable Next button
      m_NextButton.EnableWindow(TRUE);
      m_NextButton.SetFocus();

      CString str;
      str.Format("%ld frames grabbed, at a frame rate of %.2f frames/sec (%.2f ms/frame). "
                 "Click Next to end. ",
                 m_nProcessingCount, 
                 m_nProcessingCount / m_dTime, 
                 1000.0 * m_dTime / m_nProcessingCount);
      m_strCommentText = str;
      UpdateData(FALSE);
      }

   return 0;
}

/////////////////////////////////////////////////////////////////////////////
// OnClose message handler
// Here in case the thread is running while the program exists.
// Instructs the thread to exit unconditionally at the first chance.
void CExampleDlg::OnClose() 
{
   // Instruct the processing thread to terminate if necessary
   m_bKillProcessing = TRUE;
	
	CDialog::OnClose();
}
