// MGrabSequence example

// Synopsis:   This example grabs a sequence of images using a hook 
//             function and plays it back.
//
//             Note this assumes that the copy operation is shorter 
//             than a grab and that the PC has sufficient bandwith 
//             to support the operations.

// NOTE: Check the history logs to determine which properties were set at
//       design time.

// ExampleDlg.cpp: implementation file
//

#include "stdafx.h"
#include "ExampleDlg.h"
#include "AVIFileHandler.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// Location of AVI file
const char EXAMPLE_AVI_PATH[] = "..\\..\\..\\..\\..\\images\\seq.avi";  


// Custom Windows message 
const int WM_PLAYBACKWORKFUNCTIONEND = WM_USER + 1;


BEGIN_MESSAGE_MAP(CExampleDlg, CDialog)
   //{{AFX_MSG_MAP(CExampleDlg)
   ON_BN_CLICKED(IDC_NEXTBUTTON, OnNextbutton)
   ON_WM_CLOSE()
   ON_BN_CLICKED(IDC_OPT_UNCOMPRESSED, OnOptUncompressed)
   ON_MESSAGE(WM_PLAYBACKWORKFUNCTIONEND, OnPlaybackWorkFunctionEnd)
   ON_BN_CLICKED(IDC_OPT_JPEG_LOSSY_COMPRESSION, OnOptJPEGLossyCompression)
   //}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// The one and only CExampleApp object
//
CExampleApp theApp;


/////////////////////////////////////////////////////////////////////////////
// CExampleDlg dialog
//
CExampleDlg::CExampleDlg(CWnd* pParent /*=NULL*/)
   : CDialog(CExampleDlg::IDD, pParent)
{
   //{{AFX_DATA_INIT(CExampleDlg)
   m_strCommentText = _T("");
   //}}AFX_DATA_INIT

   m_nStepCount = 1;
   m_bUseCompression = FALSE;
   m_dAverageFramePlaybackTime = 0.0;

   m_aSequenceImages.SetSize(MAXIMUM_NUMBER_OF_FRAMES);
}

CExampleDlg::~CExampleDlg()
{
}


/////////////////////////////////////////////////////////////////////////////
// CExampleDlg data exchange
//
void CExampleDlg::DoDataExchange(CDataExchange* pDX)
{
   CDialog::DoDataExchange(pDX);
   //{{AFX_DATA_MAP(CExampleDlg)
   DDX_Control(pDX, IDC_OPT_JPEG_LOSSY_COMPRESSION, m_optJPEGLossyCompression);
   DDX_Control(pDX, IDC_OPT_UNCOMPRESSED, m_optUncompressed);
   DDX_Control(pDX, IDC_CHK_SAVE_FILE, m_chkSaveFile);
   DDX_Control(pDX, IDC_NEXTBUTTON, m_NextButton);
   DDX_Text(pDX, IDC_COMMENTTEXT, m_strCommentText);
   DDX_Control(pDX, IDC_IMAGE_BUFFER, m_ImageBuffer);
   DDX_Control(pDX, IDC_DISPLAY, m_Display);
   DDX_Control(pDX, IDC_DIGITIZER, m_Digitizer);
   DDX_Control(pDX, IDC_SYSTEM, m_System);
   DDX_Control(pDX, IDC_APPLICATION, m_Application);
   //}}AFX_DATA_MAP
}


/////////////////////////////////////////////////////////////////////////////
// CExampleDlg message handlers
//
BOOL CExampleDlg::OnInitDialog()
{
   CDialog::OnInitDialog();

   // Select defaults for the example: uncompressed images and save to file.
   m_optUncompressed.SetCheck(TRUE);
   m_chkSaveFile.SetCheck(TRUE);

   m_Digitizer.SetScaleX(0.5);
   m_Digitizer.SetScaleY(0.5);

   // Disable the JPEG lossy compression button if compression is not supported
   if (!m_System.GetCompressionSupported())
      {
      m_optJPEGLossyCompression.EnableWindow(FALSE);
      }
   
   // Resize the dialog 
   AdjustDialog();


   ///////
   // Step 1: Resize and allocate the ImageBuffer, and start the grab.
   m_ImageBuffer.SetSizeX((long)(m_Digitizer.GetSizeX() * m_Digitizer.GetScaleX()));
   m_ImageBuffer.SetSizeY((long)(m_Digitizer.GetSizeY() * m_Digitizer.GetScaleY()));
   m_ImageBuffer.Allocate();
   m_Digitizer.GrabContinuous();

   // Print message
   m_strCommentText = "This will record a sequence in a loop.  "
                      "Click Next to continue.  ";

   UpdateData(FALSE);
   return TRUE;  
}


/////////////////////////////////////////////////////////////////////////////
// OnNextButton message handler.
// This function drives the example through each step, performing appropriate
// actions
//
void CExampleDlg::OnNextbutton() 
{
   // Increments the step counter and updates text in the dialog 
   AdjustStepNumber();

   switch (m_nStepCount)
      {
      ///////
      // Step 2: Grab, save, and play the sequence
      case 2:
         {
         // Disable the Next button.
         m_NextButton.EnableWindow(FALSE);

         // Stop continuous grab
         m_Digitizer.Halt();

         // Disable compression buttons
         m_optJPEGLossyCompression.EnableWindow(FALSE);
         m_optUncompressed.EnableWindow(FALSE);

         // Print message
         m_strCommentText = "Allocating sequence images...  ";
         UpdateData(FALSE);
         
         // The compression algorithm used depends on the system and the digitizer
         LONG CompressiontType = mJPEGLossy;
         if ((m_System.GetBoardType().Find("Compression") != -1) && 
             (m_Digitizer.GetScanMode() == mInterlace))
            {
            CompressiontType = mJPEGLossyInterlaced;
            }

         // Allocate all the sequence images
         BOOL bContinueAllocatingImages = TRUE;
         m_nNumberOfFrames = 0;

         while (bContinueAllocatingImages)
            {
            // Create a new image control
            IImage NewImage(IImage::NewInstance());

            // Set its properties
            NewImage.SetCanGrab(TRUE);
            NewImage.SetCanDisplay(FALSE);
            NewImage.SetNumberOfBands(m_ImageBuffer.GetNumberOfBands());
            NewImage.SetSizeX(m_ImageBuffer.GetSizeX());
            NewImage.SetSizeY(m_ImageBuffer.GetSizeY());

            // If compression is wanted, set it the type and Q factor
            if (m_bUseCompression)
               {
               NewImage.SetCompressionType(CompressiontType);     
               NewImage.GetJPEGAlgorithm().SetQFactor(Q_FACTOR);
               }

            // Allocate the new image; if this fails, we asume we are 
            // out of memory and we exit the allocation loop
            try 
               {
               NewImage.Allocate();

               m_aSequenceImages[m_nNumberOfFrames] = NewImage;
               m_nNumberOfFrames++;
               
               if (m_nNumberOfFrames == MAXIMUM_NUMBER_OF_FRAMES)
                   bContinueAllocatingImages = FALSE;
               }

            catch(COleDispatchException* e)
               {
               bContinueAllocatingImages = FALSE;
               e->Delete();
               }
            }

         // Print waiting message
         m_strCommentText.Format("Grabbing a sequence of %d images, please wait... ",
                                 m_nNumberOfFrames);
         UpdateData(FALSE);

         // Synchronize the timer and start the grab
         m_Digitizer.SetImage(m_aSequenceImages[0]);
         m_Digitizer.Grab();
         m_Digitizer.Grab();
         m_Application.GetTimer().Reset();

         for (int i=1; i<m_nNumberOfFrames; i++)
            {
            m_Digitizer.SetImage(m_aSequenceImages[i]);
            m_Digitizer.Grab();
            }

         // Wait for the last frame to be grabbeb, and then get the time it took
         // to grab the sequence
         m_Digitizer.GrabWait(mGrabEnd);
         m_dTime = m_Application.GetTimer().Read();

         // Disable the check button indicating whether the AVI should be saved
         m_chkSaveFile.EnableWindow(FALSE);
         
         // Save the sequence into an AVI file, according the check button
         if (m_chkSaveFile.GetCheck())
            {
            // Print saving message
            m_strCommentText.Format("Saving and re-loading the sequence, please wait... ");
            UpdateData(FALSE);

            IAVIFileHandler AVIHandler = IAVIFileHandler::NewInstance();

            AVIHandler.Open(EXAMPLE_AVI_PATH, aviFileFormatDefault, aviCreate);
            AVIHandler.Write(m_aSequenceImages, m_nNumberOfFrames / m_dTime);
            AVIHandler.Close();
     
            // Clear all buffers in the sequence
            for (int i=1; i<m_nNumberOfFrames; i++)
               m_aSequenceImages[i].Clear();

            // Restore the sequence from the file
            AVIHandler.Open(EXAMPLE_AVI_PATH, aviFileFormatDefault, aviRead);
            AVIHandler.Read(m_aSequenceImages, 0, m_nNumberOfFrames);
            AVIHandler.Close();

            }

         // Start playback thread
         m_bStopPlayback = FALSE;
         m_bKillPlayback = FALSE;
         AfxBeginThread(PlaybackThreadFunction, this);

         // Print statistics
         m_strCommentText.Format("Playing the sequence; %d frames grabbed at a frame rate of %.2f frames/sec "
                                 "(%.2f ms/frame).  "
                                 "Click Next to continue.  ",
                                 m_nNumberOfFrames,
                                 m_nNumberOfFrames / m_dTime,
                                 1000.0 * m_dTime / m_nNumberOfFrames);

         // Reenable the  Next button.
         m_NextButton.EnableWindow(TRUE);

         break;
         }

      ///////
      // Step 3: Stop the playback and print statistics
      case 3:
         {
         // Instruct the processing thread to stop its loop
         InterlockedExchange (&m_bStopPlayback, TRUE);

         // To be continued in OnPlaybackWorkFunctionEnd...
         break;
         }

      ///////
      // Last Step: End the application
      default:
         {
         // Quit application
         EndDialog(IDOK);

         break;
         }
      }

   UpdateData(FALSE);
}


/////////////////////////////////////////////////////////////////////////////
// PlaybackThreadFunction.
// This function controls the playback thread
//
UINT CExampleDlg::PlaybackThreadFunction(LPVOID pParam)
{
   CExampleDlg *pthis = (CExampleDlg *) pParam;

   // Call CExampleDlg's playback loop
   //
   // Enclose ActiveMIL calls in a try-catch statement in order
   // to prevent ActiveMIL errors to cause an uncaught 
   // exception message.  MFC does not catch exceptions thrown by
   // ActiveX controls outside of the message loop.
   try
      {
      pthis->PlaybackWorkFunction();
      }
   catch (COleDispatchException *e)
      {
      // Display error message
      e->ReportError();

      // Always call Delete() on caught exceptions
      e->Delete();

      // Kill application
      ExitProcess(1);
      }
   
   // Post a custom Windows message to inform the main UI thread of
   // the playback completion.
   //
   // A custom message handler will then be triggered and 
   // perform the remaining tasks of the current example step.
   //
   // This is the preferred way of communicating with worker threads in
   // MFC. An alternative way would be to use synchronised wait functions
   // such as WaitForSingleObject(MyThreadHandle) to wait for thread 
   // termination. But this way, UI components would be frozen and the 
   // application would stop responding. 
   //
   // Hence, we recommend that all lengthy processing be performed in a work 
   // thread in such a way as not to block the main UI thread.
   pthis->PostMessage(WM_PLAYBACKWORKFUNCTIONEND);

   return 0;
}


/////////////////////////////////////////////////////////////////////////////
// PlaybackWorkFunction() event handler.
// This function performs the real work of the playback thread
//
void CExampleDlg::PlaybackWorkFunction()
{
   IApplicationTimer Timer(m_Application.GetTimer());

   double dTimeWait     = 0;
   double dTemp         = 0;
   double dTotalTime    = 0;
   long   nFrameNumber  = 0;
   long   nFramesPlayed = 0;


   while (! m_bStopPlayback && ! m_bKillPlayback)
      {
      // Reset timer and copy next image to display
      Timer.Reset();
      m_ImageBuffer.Copy(m_aSequenceImages[nFrameNumber]);

      dTemp = Timer.Read();

      dTotalTime += dTemp;

      // Compute how much time we must still wait for this frame
      dTimeWait = m_dTime / m_nNumberOfFrames - dTemp;

      dTotalTime += dTimeWait;

      // Wait for remaining time
      Timer.Wait(dTimeWait);

      // Adjust frame number and number of frames played
      nFrameNumber = ++nFrameNumber % m_nNumberOfFrames;
      nFramesPlayed++;

      // Update the average time of an iteration (sec per frame)
      m_dAverageFramePlaybackTime = dTotalTime / nFramesPlayed;
      }
}


/////////////////////////////////////////////////////////////////////////////
// OnProcessingWorkFunctionEnd message handler
// This function handles WM_PROCESSINGWORKFUNCTIONEND custom Windows message
// posted by the processing thread control function at end of processing
//
LONG CExampleDlg::OnPlaybackWorkFunctionEnd( UINT, LONG )
{
   // ...sequel of "Step 3: Stop the playback and print statistics"
   if (! m_bKillPlayback)
      {
      // Print statistics
      m_strCommentText.Format("%d frames replayed at a frame rate of %.2f frames/sec "
                              "(%.2f ms/frame).  "
                              "Click Next to end.  ",
                              m_nNumberOfFrames,
                              1.0 / m_dAverageFramePlaybackTime,
                              1000.0 * m_dAverageFramePlaybackTime);

      UpdateData(FALSE);
      }

   return 0;
}



/////////////////////////////////////////////////////////////////////////////
// OnClose message handler
// Here in case the thread is running while the program exists.
// Instructs the thread to exit unconditionally at the first chance.
void CExampleDlg::OnClose() 
{
   // Instruct the playback thread to terminate if necessary
   m_bKillPlayback = TRUE;
   
   CDialog::OnClose();
}

void CExampleDlg::OnOptJPEGLossyCompression() 
{
   m_bUseCompression = TRUE;

   // There is a limitation on the Meteor II board with MJPEG compression module 
   // that prevents digitizer scaling when grabbing with compression.

   if (m_System.GetBoardType().Find("Compression") != -1)
      {
      // Change the Digitizer scale and Resize the ImageBuffer
      if (m_ImageBuffer.GetIsAllocated())
         m_ImageBuffer.Free();
      if (m_Digitizer.GetIsAllocated())
         m_Digitizer.Free();
    
      m_Digitizer.SetScaleX(1.0);
      m_Digitizer.SetScaleY(1.0);

      m_ImageBuffer.SetSizeX((long)(m_Digitizer.GetSizeX() * m_Digitizer.GetScaleX()));
      m_ImageBuffer.SetSizeY((long)(m_Digitizer.GetSizeY() * m_Digitizer.GetScaleY()));
    
      // Resize the dialog 
      AdjustDialog();

      m_ImageBuffer.Allocate();
      m_Digitizer.Allocate();
      m_Digitizer.GrabContinuous();
      }
}

void CExampleDlg::OnOptUncompressed() 
{
   m_bUseCompression = FALSE;

   // There is a limitation on the Meteor II board with MJPEG compression module 
   // that prevents digitizer scaling when grabbing with compression.

   if (m_System.GetBoardType().Find("Compression") != -1)
      {
      // Change the Digitizer scale and Resize the ImageBuffer
      if (m_ImageBuffer.GetIsAllocated())
         m_ImageBuffer.Free();
      if (m_Digitizer.GetIsAllocated())
         m_Digitizer.Free();
    
      m_Digitizer.SetScaleX(0.5);
      m_Digitizer.SetScaleY(0.5);

      m_ImageBuffer.SetSizeX((long)(m_Digitizer.GetSizeX() * m_Digitizer.GetScaleX()));
      m_ImageBuffer.SetSizeY((long)(m_Digitizer.GetSizeY() * m_Digitizer.GetScaleY()));

      // Resize the dialog 
      AdjustDialog();

      m_ImageBuffer.Allocate();
      m_Digitizer.Allocate();
      m_Digitizer.GrabContinuous();
      }
}
