; File name: mobjtrac.cl
; Synopsis:  This program shows you how to track a unique object
;            on a plain background using pattern recognition. It
;            allocates a model in the field of view of the camera
;            and finds it in a loop. It also prints the coordinates
;            and draws a box around it.
;
;            Note: This program assumes that you have a camera
;            plugged into channel 0 of a monochrome digitizer.
 


; Disable the automatic printing of the return values of commands
silence 1

; Target image specifications. 
macro IMAGE_WIDTH   512
macro IMAGE_HEIGHT  480

; Given model width and height. 
macro MODEL_WIDTH    128
macro MODEL_HEIGHT   128
macro MODEL_POS_X_INIT    256.0
macro MODEL_POS_Y_INIT    240.0


  MIL_ID MilApplication          ; Application identifier.  
  MIL_ID MilSystem               ; System identifier.       
  MIL_ID MilDisplay              ; Display identifier.      
  MIL_ID MilDigitizer            ; Camera identifier. 
  MIL_ID MilImage                ; Image buffer identifiers. 
  MIL_ID MilSubImage             ; Sub-Image buffer identifier. 
  MIL_ID Model                   ; Model identifier.
  MIL_ID Result                  ; Result identifier. 
  long   Width                  ; Model size. Might get changed... 
  setWidth    MODEL_WIDTH       
  long   Height                 ; ...if not large enough. 
  setHeight   MODEL_HEIGHT
  long   PosX                   ; Model position. 
  long   PosY
  long   Found                  ; Number of found model. 
  double OrgX                   ; Original center of model. 
  double OrgY
  double x                      ; Result variables. 
  setx   MODEL_POS_X_INIT
  double y
  sety   MODEL_POS_Y_INIT
  setnbr x1 (x)
  setnbr y1 (y)
  long   AllocError             ; Allocation error variable. 
  long   castx
  long   casty
  double Score
  macro  MODEL_MIN_MATCH_SCORE 60.0 ; the object found (in percent). 

   ; Allocate defaults. 
   MappAllocDefault M_SETUP &MilApplication &MilSystem &MilDisplay &MilDigitizer &MilImage

  if ([MbufInquire MilImage M_SIZE_BAND M_NULL] != 1)  
     {
     echo "The example does not work as it is with a color buffer."
     echo "A monochrome buffer will be used instead."

     setnbr SizeX [MbufInquire MilImage M_SIZE_X M_NULL]
     setnbr SizeY [MbufInquire MilImage M_SIZE_Y M_NULL]

     MbufFree MilImage
     MbufAlloc2d MilSystem [getnbr SizeX] [getnbr SizeY] SizT Attr &MilImage

     MbufClear MilImage 0
     MdispSelect MilDisplay MilImage                  
     }

   ; Restrict the region to be processed to a sub-image. 
   MbufChild2d MilImage 0 0 IMAGE_WIDTH IMAGE_HEIGHT &MilSubImage

   ; Continuously grab and find. 
   echo "Place the model to find in the marked rectangle."
   echo "and press a key to continue."

   while ( [keyhit] == 0 )
      {
      ; Grab a target image. 
      MdigGrab MilDigitizer MilSubImage
 
      ; Draw a box around the position of the model to define. 
      setnbr xstart ( x-((MODEL_WIDTH/2)-2)  )
      setnbr ystart ( y-((MODEL_HEIGHT/2)-2) )
      setnbr xend   ( x+((MODEL_WIDTH/2)+1)  )
      setnbr yend   ( y+((MODEL_HEIGHT/2)+1) )
 
      MgraRect M_DEFAULT MilSubImage [getnbr xstart] [getnbr ystart] [getnbr xend] [getnbr yend] 
      }

   ; Allocate normalized grayscale type model. 
   MpatAllocModel MilSystem MilSubImage ([getnbr x1]-((MODEL_WIDTH/2)-1) ) ([getnbr y1]-((MODEL_HEIGHT/2)-1) ) MODEL_WIDTH MODEL_HEIGHT M_NORMALIZED &Model

   setPosX  (x-((MODEL_WIDTH/2)-1) )
   setPosY  (y-((MODEL_HEIGHT/2)-1) )

   ; Allocate result. 
   MpatAllocResult MilSystem 1 &Result

   ; Check for a successful allocations. 
   MappGetError M_GLOBAL &AllocError

   if (AllocError==0)
      {
      ; Draw box around the model. 
      MgraRect M_DEFAULT MilSubImage (PosX-1) (PosY-1) (PosX+MODEL_WIDTH) (PosY+MODEL_HEIGHT)
      }
   else
      {
      echo "Error: Allocation or automatic model definition failed."
      pause "Press <Enter> to end."

      ; Free workspace, model and result buffers. 
      if (Result!=0) { MpatFree Result }
      if (Model!=0) { MpatFree Model }
      if (MilSubImage!=0) { MbufFree MilSubImage }

      ; Release defaults. 
      MappFreeDefault MilApplication MilSystem MilDisplay MilDigitizer MilImage

      return
      }

   ; Set minimum acceptance for search. 
   MpatSetAcceptance Model MODEL_MIN_MATCH_SCORE

   ; Set speed. 
   MpatSetSpeed Model M_HIGH

   ; Set accuracy. 
   MpatSetAccuracy Model M_LOW

   ; Preprocess model. 
   MpatPreprocModel MilSubImage Model M_DEFAULT

   ; Inquire about center of model. 
   MpatInquire Model M_ORIGINAL_X &OrgX
   MpatInquire Model M_ORIGINAL_Y &OrgY

   ; Continuously grabbing and finding. 
   echo "Continuously finding the defined model in the live image..."
         
   while ([keyhit]==0) {
      ; Grab a target image. 
      MdigGrab MilDigitizer MilSubImage

      ; Find model. 
      MpatFindModel MilSubImage Model Result

      ; Get results. 
      MpatGetNumber Result &Found
      MpatGetResult Result M_POSITION_X &x
      MpatGetResult Result M_POSITION_Y &y
      MpatGetResult Result M_SCORE &Score

      ; setnbr x1 (x)
      ; setnbr y1 (y)
     
      if (Found != 0)
         {
         ; Draw a box around the found model and print
         ; the found position and score.
         setnbr xstart (x-((MODEL_WIDTH/2)-1))
         setnbr ystart (y-((MODEL_HEIGHT/2)-1))
         setnbr xend (x+(MODEL_WIDTH/2))
         setnbr yend (y+(MODEL_HEIGHT/2))
         setnbr xmod (x)
         setnbr ymod (y)
         setnbr scoremod (Score)
         MgraRect M_DEFAULT MilSubImage [getnbr xstart] [getnbr ystart] [getnbr xend] [getnbr yend]

         echoln "Model position X="
         echoln [dbltostr [getnbr xmod] 4]
         echoln " ,Y="                  
         echoln [dbltostr [getnbr ymod] 4]
         echoln ", score="
         echo   [dbltostr [getnbr scoremod] 4]
         }

      else
         {
         echo "Model not found! (score<60.0%)"
         }
      } 
   echo "done."

   ; Free workspace, model and result buffers. 
   if (Result!=0) { MpatFree Result }
   if (Model!=0) { MpatFree Model }
   if (MilSubImage!=0) { MbufFree MilSubImage }

   MappFreeDefault MilApplication MilSystem MilDisplay MilDigitizer MilImage

   ; Enable the automatic printing of the return values of commands
silence 0

