;File name: mocrfont.c
;Synopsis:  This program create a custom OCR font, set its constraints
;           and use it to read a string drawn in the image. The string
;           read is then printed to the screen and the calibrated font
;           saved to disk.

; Disable the automatic printing of the return values of commands
silence 1

; Typical reading specifications.
macro STRING_TO_READ           "KF419N"
macro STRING_SCORE_MIN         50.0
macro STRING_SCALE             2.0

; Font specifications.
macro F_C_LIST           "FKLMN0123456789"
macro F_C_NUM            15
macro F_C_BOX_X          16
macro F_C_BOX_Y          32
macro F_C_OFFSET_X       1
macro F_C_OFFSET_Y       6
macro F_C_X              14
macro F_C_Y              18
macro F_C_THICKNESS      4
macro F_NUM_C_TO_READ    6
macro F_C_FOREGROUND     M_FOREGROUND_WHITE
macro F_FILE_NAME        "\custom.mfo"

; Work region specifications.
macro W_R_POS_X  20
macro W_R_POS_Y  80
macro W_R_WIDTH  (( F_C_BOX_X * F_C_NUM + 1) * STRING_SCALE)
macro W_R_HEIGHT (( F_C_BOX_Y * 2) * STRING_SCALE)

; Image background color.
macro BACKGROUND_COLOR   128

MIL_ID MilApplication              ; Application identifier.  
MIL_ID MilSystem                   ; System identifier.       
MIL_ID MilDisplay                  ; Display identifier.      
MIL_ID MilImage                    ; Image buffer identifier. 
MIL_ID MilSubImage                 ; Sub-image buffer identifier. 
MIL_ID OcrFont                     ; OCR font identifier.         
MIL_ID OcrResult                   ; OCR result buffer identifier.
double Score                       ; Reading score.               
setstr String "1234567"            ; String for the read characters.

; Allocate defaults
   MappAllocDefault M_SETUP &MilApplication &MilSystem &MilDisplay M_NULL &MilImage

; Restrict the source image to the work region.
   MbufChild2d MilImage W_R_POS_X W_R_POS_Y W_R_WIDTH W_R_HEIGHT &MilSubImage

; Set the foreground color for graphics to white
   MgraColor M_DEFAULT 255

; Draw a repesentation of all the characters of the new font to create.
   MbufClear MilSubImage 0
   MgraFont M_DEFAULT M_FONT_DEFAULT_LARGE
   MgraText M_DEFAULT MilSubImage 1 0 F_C_LIST

; Pause to show the characters of the font.
   echo "A custom OCR font will be created from the characters drawn"
   echo "in the displayed image.
   pause "Press <Enter> to continue."

; Allocate a new empty OCR font.
   MocrAllocFont M_DEFAULT M_DEFAULT F_C_NUM F_C_BOX_X F_C_BOX_Y F_C_OFFSET_X F_C_OFFSET_Y F_C_X F_C_Y F_C_THICKNESS F_NUM_C_TO_READ F_C_FOREGROUND &OcrFont

; Copy the character representation to the font.
   MocrCopyFont MilSubImage OcrFont M_COPY_TO_FONT F_C_LIST

; Set character constraints for each position of the string to read.
   MocrSetConstraint OcrFont 0 M_LETTER  "K"          ; Must be K.      
   MocrSetConstraint OcrFont 1 M_LETTER  "FKLMN"      ; Any letter.     
   MocrSetConstraint OcrFont 2 M_DIGIT   "0123456789" ; Any digit.      
   MocrSetConstraint OcrFont 3 M_DIGIT   "12"         ; Must be 1 or 2  
   MocrSetConstraint OcrFont 4 M_DIGIT   "0123456789" ; Any digit.      
   MocrSetConstraint OcrFont 5 M_DEFAULT "FKLMN"      ; Any character   

; Set the target image character scale for the font manually.
   MocrControl OcrFont M_TARGET_CHAR_SIZE_X (F_C_X * STRING_SCALE)
   MocrControl OcrFont M_TARGET_CHAR_SIZE_Y (F_C_Y * STRING_SCALE)
   MocrControl OcrFont M_TARGET_CHAR_SPACING (F_C_BOX_X * STRING_SCALE)

; Draw a typical string respecting the font and its constraints
; but at a bigger scale.

   MbufClear MilSubImage 0
   MgraFontScale M_DEFAULT STRING_SCALE STRING_SCALE
   MgraText M_DEFAULT MilSubImage  (STRING_SCALE + 0.5) 0 STRING_TO_READ

; Pause to show the string to read.
   echo     
   echo "A typical string with a bigger scale will be read using"
   echo "the new custom font and the result will be printed."
   pause "Press <Enter> to continue."

; Allocate an OCR result buffer.
   MocrAllocResult MilSystem M_DEFAULT &OcrResult

; Read the string.
   MocrReadString MilSubImage OcrFont OcrResult

; Get the string and its reading score.
   MocrGetResult OcrResult M_STRING [getstrptr String]
   MocrGetResult OcrResult M_SCORE  &Score

; Print the result.
   echo  "The string read is:"
   echo  [getstr String]
   echoln  "Score: "
   echoln  [dbltostr Score 4]
   echo    " %"

; Save the custom font if the reading score was sufficent.
   if  ( Score > STRING_SCORE_MIN )
      {
	  setstr str_font [addstr imagepath F_FILE_NAME]
      MocrSaveFont [getstr str_font] M_SAVE OcrFont
      echo "Read sucessful calibrated OCR font was saved."
      }
   else 
      {
      echo "Error: Read score too low custom OCR font not saved."
      }
   pause "Press <Enter> to end."
   
; Free all allocations.
   MocrFree OcrFont
   MocrFree OcrResult
   MbufFree MilSubImage
   MappFreeDefault MilApplication MilSystem MilDisplay M_NULL MilImage

; Enable the automatic printing of the return values of commands
silence 0
