/* --------------------------------------------------------------- */
/* File name: mdispovr.c
 * Synopsis:  This program shows how to display an image while doing 
 *	      text and graphics annotations on top of it using MIL 
 *	      graphic functions and windows GDI drawing (under Windows). 
 *	      If the target system supports grab, the annotations are 
*	      done on top of live video.
 *
 *            Note that if true VGA overlay (color keying) is supported 
 *            by the target system, the annotations are done in the VGA 
 *            overlay surface, otherwhise, overlay is simulated by MIL 
 *            and pseudo-live grab will be used if grabbing.
 *
 */

#include <stdio.h>  
#include <mil.h>
#include <string.h>
#if !M_MIL_USE_DOS_32
#include <windows.h>
#endif

/* Target MIL image file specifications. */  
#define IMAGE_FILE        M_IMAGE_PATH MT("board.mim")

/* Title to put on the display window */
#define WINDOW_TITLE MT("Custom Title")

/* Overlay update function prototype */
void MyOverlayDraw(MIL_ID System, MIL_ID Display, long WindowedMode);

void main(void)
{ 
   MIL_ID MilApplication,            /* Application identifier.   */
          MilSystem,                 /* System identifier.        */
          MilDisplay,                /* Display identifier.       */
          MilDigitizer=0,            /* Digitizer identifier.     */
          MilImage;                  /* Image identifier.         */

   long   WindowedMode;

   /* Allocate defaults */
   MappAllocDefault(M_SETUP, &MilApplication, &MilSystem, &MilDisplay, 
                    M_NULL, M_NULL);
       
   if(MdispInquire(MilDisplay, M_DISPLAY_MODE, M_NULL)==M_WINDOWED)
      WindowedMode=M_TRUE;
   else
      WindowedMode=M_FALSE;  

   /* If the system can grab. */
   if (MsysInquire(MilSystem, M_DIGITIZER_NUM, M_NULL))
      {
      /* Allocate a digitizer. */
      MdigAlloc(MilSystem, M_DEFAULT, M_CAMERA_SETUP, M_DEFAULT, &MilDigitizer);
     
      /* Allocate a grab image that matches the digitizer input format. */
      MbufAllocColor(MilSystem,
                     MdigInquire(MilDigitizer,M_SIZE_BAND,M_NULL),
                     MdigInquire(MilDigitizer,M_SIZE_X,M_NULL),
                     MdigInquire(MilDigitizer,M_SIZE_Y,M_NULL),
                     8+M_UNSIGNED, 
                     M_IMAGE+M_DISP+M_PROC+M_GRAB,
                     &MilImage);
                     
      /* Clear the display image buffer. */ 
      MbufClear(MilImage, 0); 
      }
   else
      {
      /* Restore source image into an automatically allocated image buffer. */
      MbufRestore(IMAGE_FILE, MilSystem, &MilImage);
      }
     
   /* Change display window title. */
   if (WindowedMode)
      MdispControl(MilDisplay, M_WINDOW_TITLE_NAME, (long)WINDOW_TITLE);

   /* Display the image buffer. */
   MdispSelect(MilDisplay, MilImage);
     
   /* Draw text and graphics in the overlay buffer */
   MyOverlayDraw(MilSystem,MilDisplay,WindowedMode);
                     
   /* If the system can grab. */
   if (MilDigitizer)
      {
      /* Grab continuously in the image on the display. */
      MdigGrabContinuous(MilDigitizer, MilImage);
      }

   /* Pause to show the image. */ 
   printf("This program shows how to display an image and how to do\n");
   printf("overlay annotations in a MIL display.\n");
   printf("Press <Enter> to continue.\n");  
   getchar();

   /* Stop the continuous grab and free digitizer (if needed). */
   if (MilDigitizer)
      {
      MdigHalt(MilDigitizer);
      MdigFree(MilDigitizer);
     
      /* Pause to show the result. */
      printf("\nDisplaying the last grabbed image.\n");
      printf("Press <Enter> to end.\n");
      getchar();
      }
  
   /* Deselect the image from the display. */
   MdispDeselect(MilDisplay, MilImage);

   /* Free image. */
   MbufFree(MilImage);
      
   /* Free defaults allocations. */
   MappFreeDefault(MilApplication, MilSystem, MilDisplay, M_NULL, M_NULL);
}


/* --------------------------------------------------------------- */
/* Name:      MyOverlayDraw
 * Synopsis:  This function enables and writes graphics into the
 *            overlay buffer. 
 */            
void MyOverlayDraw(MIL_ID MilSystem, MIL_ID MilDisplay, long WindowedMode)
{
   MIL_ID         MilOverlayImage;
   long           ImageWidth, ImageHeight;
   long           TransparentColor;

   /* Prepare overlay buffer. */
   /***************************/

   /* Disable overlay display while preparing the data. */
   MdispControl(MilDisplay, M_WINDOW_OVR_SHOW, M_DISABLE);
        
   /* Enable writing Overlay graphics on top of display buffer. */
   MdispControl(MilDisplay, M_WINDOW_OVR_WRITE, M_ENABLE);
        
   /* Inquire the Overlay buffer associated with the displayed buffer. */
   MdispInquire(MilDisplay, M_WINDOW_OVR_BUF_ID, &MilOverlayImage);

   /* Inquire the current keying color. */
   MdispInquire(MilDisplay, M_KEY_COLOR, &TransparentColor);

   /* Clear the overlay buffer with the keying color. */
   MbufClear(MilOverlayImage, TransparentColor); 
   
   /* Enable the overlay display. */
   MdispControl(MilDisplay, M_WINDOW_OVR_SHOW, M_ENABLE);
   
   /* Draw MIL monochrome overlay annotation */
   /******************************************/

   /* Inquire MilOverlayImage size x and y. */
   ImageWidth  = MbufInquire(MilOverlayImage,M_SIZE_X,M_NULL);
   ImageHeight = MbufInquire(MilOverlayImage,M_SIZE_Y,M_NULL);
      
   /* Set graphic text to transparent background. */
   MgraControl(M_DEFAULT, M_BACKGROUND_MODE, M_TRANSPARENT);

   /* Set drawing color to white. */
   MgraColor(M_DEFAULT, 255);

   /* Print a string in the overlay image buffer. */
   MgraText(M_DEFAULT, MilOverlayImage, ImageWidth/9, ImageWidth/6,    MT(" -------------------- "));
   MgraText(M_DEFAULT, MilOverlayImage, ImageWidth/9, ImageWidth/6+25, MT(" - MIL Overlay Text - "));
   MgraText(M_DEFAULT, MilOverlayImage, ImageWidth/9, ImageWidth/6+50, MT(" -------------------- "));
   
   /* Draw MIL color overlay annotation */
   /*************************************/

   /* If the overlay buffer is monochrome, associate a pseudo-color */
   /* LUT to the overlay display and draw color annotations.        */
   if (MbufInquire(MilOverlayImage, M_SIZE_BAND, M_NULL) == 1)
      {
      /* Local variables and arrays. */
      MIL_ID         MILLut, MILLutChildLow, MILLutChildHigh;
      
      /* LUT data for PSEUDO-COLOR:          R    G    B     Entry:    Color: */
      static unsigned char LutLow[30] =  {   0,   0,   0, // Index 0 = black 
                                           128,   0,   0, // Index 1 = dark-red
                                             0, 128,   0, // Index 2 = dark-green
                                           128, 128,   0, // Index 3 = dark-yellow
                                             0,   0, 128, // Index 4 = dark-blue
                                           128,   0, 128, // Index 5 = dark-magenta
                                             0, 128, 128, // Index 6 = dark-cyan
                                           192, 192, 192, // Index 7 = bright-grey
                                           192, 220, 192, // Index 8 = lite-green
                                           166, 202, 240  // Index 9 = lite-blue
                                         };
      static unsigned char LutHigh[30] = { 255, 251, 240, // Index 246 = lite-white
                                           160, 160, 164, // Index 247 = lite-grey
                                           128, 128, 128, // Index 248 = dark-grey
                                           255,   0,   0, // Index 249 = red
                                             0, 255,   0, // Index 250 = green
                                           255, 255,   0, // Index 251 = yellow
                                             0,   0, 255, // Index 252 = blue
                                           255,   0, 255, // Index 253 = magenta
                                             0, 255, 255, // Index 254 = cyan
                                           255, 255, 255  // Index 255 = white
                                         };

      /* Create a grey scale LUT with pseudo-colors in the 2 extremes. */
      /* This LUT match the Windows default palette in 8 bit VGA mode  */
      /* and give the best results for grayscale image display with    */
      /* color anotations.                                             */
      MbufAllocColor(MilSystem, 3L, 256L, 1L, 8L+M_UNSIGNED, M_LUT, &MILLut);
      MbufChild2d(MILLut, 0,   0, 10,  1, &MILLutChildLow);
      MbufChild2d(MILLut, 246, 0, 10,  1, &MILLutChildHigh);
      MgenLutRamp(MILLut, 0,   0, 255, 255);
      MbufPutColor(MILLutChildLow,  M_RGB24+M_PACKED, M_ALL_BAND, &LutLow);
      MbufPutColor(MILLutChildHigh, M_RGB24+M_PACKED, M_ALL_BAND, &LutHigh);
      
      #if !M_MIL_USE_DOS_32
      /* Insert the transparent color in the LUT */
      {
      PALETTEENTRY   PalEntry;
      HDC            hDC;
      COLORREF       ColRef;
      MIL_ID         MILLutChildKey;

      hDC = GetDC(NULL);
      GetSystemPaletteEntries(hDC, TransparentColor, 1, &PalEntry);
      ColRef = RGB(PalEntry.peRed, PalEntry.peGreen, PalEntry.peBlue);
      MbufChild1d(MILLut, TransparentColor, 1, &MILLutChildKey);
      MbufPutColor(MILLutChildKey, (M_RGB24 | M_PACKED), M_ALL_BAND, &ColRef);
      ReleaseDC(NULL, hDC);

      MbufFree(MILLutChildKey);
      }
      #endif

      /* Associate the pseudo-color LUT to the overlay display. */
      MdispControl(MilDisplay, M_WINDOW_OVR_LUT, MILLut);

      /* Free the LUT. */
      MbufFree(MILLutChildHigh);
      MbufFree(MILLutChildLow);
      MbufFree(MILLut);
      
      /* Set drawing color to the green index in the pseudo-color LUT. */
      MgraColor(M_DEFAULT, 250);

      /* Print a string in the overlay image buffer. */
      MgraText(M_DEFAULT, MilOverlayImage, ImageWidth*11/18, ImageWidth/6,    MT(" -------------------- "));
      MgraText(M_DEFAULT, MilOverlayImage, ImageWidth*11/18, ImageWidth/6+25, MT(" - MIL Overlay Text - "));
      MgraText(M_DEFAULT, MilOverlayImage, ImageWidth*11/18, ImageWidth/6+50, MT(" -------------------- "));
      }
   else    
   /* If the overlay buffer is color, draw annotations only in the */
   /* selected color band to create color annotation.              */
      {
      /* Local variable. */
      MIL_ID MilOverlayImageGreen;

      /* Create a child buffer on the green component for drawing. */
      MbufChildColor(MilOverlayImage, M_GREEN , &MilOverlayImageGreen);
      
      /* Set drawing color to maximum intensity for green. */
      MgraColor(M_DEFAULT, 250);

      /* Print a string in the green component overlay image buffer. */
      MgraText(M_DEFAULT, MilOverlayImageGreen, ImageWidth*11/18, ImageWidth/6,    MT(" -------------------------- "));
      MgraText(M_DEFAULT, MilOverlayImageGreen, ImageWidth*11/18, ImageWidth/6+25, MT(" - MIL Overlay Text - "));
      MgraText(M_DEFAULT, MilOverlayImageGreen, ImageWidth*11/18, ImageWidth/6+50, MT(" -------------------------- "));
      
      /* Free the child buffer. */
      MbufFree(MilOverlayImageGreen);
      }

   #if !M_MIL_USE_DOS_32 && !M_MIL_USE_CE
   /* Draw GDI color overlay annotation */
   /*************************************/
   if (WindowedMode)
      {
      HDC   hCustomDC;
      HPEN  hpen, hpenOld;
      char  chText[80];

      /* Create a device context to draw in the overlay buffer with GDI. */
      MbufControl(MilOverlayImage, M_WINDOW_DC_ALLOC, M_DEFAULT);

      /* Inquire the device context. */
      hCustomDC = ((HDC)MbufInquire(MilOverlayImage, M_WINDOW_DC, M_NULL));
      if (hCustomDC)
         {
         /* Create a blue pen. */
         hpen=CreatePen(PS_SOLID, 1, RGB(0, 0, 255)); 
         hpenOld = SelectObject(hCustomDC,hpen);
         	
         /* Draw a cross in the overlay buffer. */
         MoveToEx(hCustomDC,0,ImageHeight/2,NULL);
         LineTo(hCustomDC,ImageWidth,ImageHeight/2);
         MoveToEx(hCustomDC,ImageWidth/2,0,NULL);
         LineTo(hCustomDC,ImageWidth/2,ImageHeight);
         
         /* Write text in the overlay buffer. */
         strcpy(chText, "GDI Overlay Text ");
         SetTextColor(hCustomDC,RGB(0, 0, 255));
         TextOut(hCustomDC,ImageWidth*3/18,ImageHeight*4/6, chText, strlen(chText));
         SetTextColor(hCustomDC,RGB(255, 0, 0));
         TextOut(hCustomDC,ImageWidth*12/18,ImageHeight*4/6, chText, strlen(chText));
         
         /* Deselect and destroy the blue pen. */
         SelectObject(hCustomDC,hpenOld);
         DeleteObject(hpen);
         }
      
      /* Delete created device context. */
      MbufControl(MilOverlayImage, M_WINDOW_DC_FREE, M_DEFAULT);
      
      /* Signal MIL that the overlay buffer was modified. */
      MbufControl(MilOverlayImage, M_MODIFIED, M_DEFAULT);
      }
   #endif
}
            
