/* File name: mmultdis.c 
 * Synopsis:  This program shows how to display more than one 
 *            image buffer at a time on a single display. It 
 *            allocates a displayable image buffer, allocates 
 *            two child buffers from it, and then uses these child
 *            buffers as the source and destination of a copy
 *            operation. It then writes text in each of the child 
 *            buffers.
 *
 *            Note: The display will be zoomed if the system's 
 *                  display supports it.
 */

#include <stdio.h> 
#include <stdlib.h> 
#include <mil.h>

/* MIL image file name. */
#define IMAGE_FILE             M_IMAGE_PATH MT("cell.mim")

/* MIL image file specifications. */
#define IMAGE_WIDTH            128L
#define IMAGE_HEIGHT           240L
#define IMAGE_TYPE             8L+M_UNSIGNED
#define ZOOM_VALUE             2L

void main(void) 
{ 
   MIL_ID MilApplication,  /* Application identifier.              */
          MilSystem,       /* System identifier.                   */ 
          MilDisplay,      /* Display identifier.                  */
          MilParentImage,  /* Image buffer identifier.             */
          MilSrcImage,     /* Source image buffer identifier.      */ 
          MilDstImage;     /* Destination image buffer identifier. */ 

   /* Allocate the defaults. */
   MappAllocDefault(M_SETUP, &MilApplication, &MilSystem,
                             &MilDisplay, M_NULL, M_NULL);

   /* Allocate a display image buffer. */
   MbufAlloc2d(MilSystem, IMAGE_WIDTH*2, IMAGE_HEIGHT,
               IMAGE_TYPE, M_IMAGE+M_DISP+M_PROC, &MilParentImage);
  
   /* Allocate two child buffers from the displayable parent buffer. */
   MbufChild2d(MilParentImage, 0L, 0L, IMAGE_WIDTH, IMAGE_HEIGHT,
                                                    &MilSrcImage); 
   MbufChild2d(MilParentImage, IMAGE_WIDTH, 0L, IMAGE_WIDTH, IMAGE_HEIGHT,
                                                             &MilDstImage);
        
   /* Clear the parent buffer. */ 
   MbufClear(MilParentImage, 0L);

   /* Display the parent buffer. */
   MdispSelect(MilDisplay, MilParentImage);

   /* Load the entire source image into the source child buffer. */ 
   MbufLoad(IMAGE_FILE, MilSrcImage);

   /* Copy the source child buffer into the destination child buffer */
   MbufCopy(MilSrcImage,MilDstImage);

   /* Write text in both children */
   MgraText(M_DEFAULT, MilSrcImage,
            IMAGE_WIDTH/4,IMAGE_HEIGHT/4, "Source" );
   MgraText(M_DEFAULT, MilDstImage,
            IMAGE_WIDTH/8,IMAGE_HEIGHT/4, "Destination" );

   /* Report on the Host screen what is being displayed. */
   printf("A copy was performed between the child on the\n");
   printf("left side of the screen and the child on the right side\n");
   printf("of the screen and text was written into each of them.\n");
   printf("Press <Enter> to continue.\n\n");
   getchar();

   #if !M_MIL_USE_DOS_32  /* Zoom not available under DOS */
   /* Report on the Host screen what is being displayed. */
   printf("Display zoomed by %ld in X and Y (if supported).\n", ZOOM_VALUE);

   /* Zoom both children by zooming the display. */
   MdispZoom(MilDisplay, ZOOM_VALUE, ZOOM_VALUE);

   /* Wait for a key */
   printf("Press <Enter> to end.\n");
   getchar();
   #endif
  
   /* Close the display. */
   MdispDeselect(MilDisplay, MilParentImage);

   /* Free all allocations. */
   MbufFree(MilDstImage);
   MbufFree(MilSrcImage);
   MbufFree(MilParentImage);
   MappFreeDefault(MilApplication, MilSystem, MilDisplay, M_NULL, M_NULL);
}


