/********************************************************************/
/*
 * File name: mnatfct.c 
 * Synopsis:  This shows the use of the MIL native mode programmer's kit. 
 *            This example shows how the user can mix MIL code and
 *            user code to create a pseudo-MIL function.
 *            This example creates a function that ADDs a constant
 *            to an image buffer.
 *            Note: The image must be 8 bit unsigned.
 */

/* headers. */
#include <stdio.h>
#include <mil.h>

#define MAX_DEPTH       8
#define DIMENSION_ERROR 1
#define IMAGE_FILE      M_IMAGE_PATH MT("Board.mim")

/* Pseudo-MIL Function definition. */
void AddConstToImage(MIL_ID MilImageId, unsigned char ConstantToAdd)
{
   MIL_ID        Func;
   long          ImageSizeX, ImageSizeY, ImagePitch, x, y, TempValue;
   unsigned char *ImageDataPtr;

   /* Prepare the start of the function and register the parameters. */
   Func = MfuncAlloc(MT("AddConstToLut"),2);
   MfuncParamId(Func,1,MilImageId,M_IMAGE,M_IN+M_OUT);
   MfuncParamChar(Func,2,ConstantToAdd);

   /* Mark the start of the function. */
   if (MfuncStart(Func))
      {
      /* Do the operation using a custom function and check to */
      /* not exceed the supported limits if required.          */
      if ((!MfuncParamCheck(Func)) || (MbufInquire(MilImageId,M_SIZE_BIT,M_NULL) == MAX_DEPTH))
         {
         /* Read the image data pointer, size X, size Y and pitch. */
         MbufInquire(MilImageId,M_HOST_ADDRESS,&ImageDataPtr);
         MbufInquire(MilImageId,M_SIZE_X,&ImageSizeX);
         MbufInquire(MilImageId,M_SIZE_Y,&ImageSizeY);
         MbufInquire(MilImageId,M_PITCH, &ImagePitch);

         /* Add the constant to the image. */
         for (y= 0; y < ImageSizeY; y++) /* For each line */
            {
            for (x= 0; x < ImageSizeX; x++) /* for each pixel */
               {
               /* Calculate the value to write */
               TempValue = (long)ImageDataPtr[x] + (long)ConstantToAdd;

               /* Write the value if no overflow else saturate */
               if (TempValue <= 0xff)
                  ImageDataPtr[x] = (unsigned char)TempValue;
               else 
                  ImageDataPtr[x] = 0xff;
               }
             /* Move pointer to the next line */
             ImageDataPtr += ImagePitch;
            }
         }
      else 
         {  
         /* Report a MIL error. */
         MfuncErrorReport(Func,M_FUNC_ERROR+DIMENSION_ERROR,
                          MT("Buffer type not supported"),
                          MT("Depth is not 8 bit."),
                          M_NULL,
                          M_NULL
                         );
         }
      }

   /* Signal to MIL that the image content was modified. */
   MbufControl(MilImageId, M_MODIFIED, M_DEFAULT);

   /* Mark the end of the function. */
   MfuncFreeAndEnd(Func);
}

/* Main to test the function. */
void main(void)
{
   MIL_ID MilApplication,    /* Application Identifier.  */
          MilSystem,         /* System Identifier.       */
          MilDisplay,        /* Display Identifier.      */
          MilImage;          /* Image buffer Identifier. */

   /* Allocate default application, system, display and image. */
   MappAllocDefault(M_COMPLETE, &MilApplication, &MilSystem,
                                &MilDisplay, M_NULL, M_NULL);

   /* Restore source image into an automatically allocated image buffer. */
   MbufRestore(IMAGE_FILE, MilSystem, &MilImage);
  
   /* Display the image. */
   MdispSelect(MilDisplay, MilImage);
   
   /* Pause */
   printf("Custom pseudo-MIL function creation and usage: \n\n");
   printf("Reference image was loaded, press a key to continue.\n");
   getchar();

   /* Process the image directly with custom Pseudo-Mil function */
   AddConstToImage(MilImage, 0x40);
   
   /* Pause */
   printf("The white level of the image was augmented using a user made\n"); 
   printf("pseudo-MIL function accessing directly the image.\n");
   printf("Press a key to terminate.\n");
   getchar();

   /* Remove the image from display. */
   MdispDeselect(MilDisplay, MilImage);

   /* Release all allocated buffers. */
   MbufFree(MilImage);

   /* Free defaults */
   MappFreeDefault(MilApplication, MilSystem, MilDisplay, M_NULL, M_NULL);
}


