/* File name: mthread.c 
 * Synopsis:  This program shows how to use different threads and synchronize 
 *            them with MIL. It creates 4 drawing threads that are used
 *            to work in 4 different regions of a display buffer.
 *
 *     Thread usage:
 *      - The main thread starts a processing thread in each of the 4 different
 *        quarters of a display buffer. The main thread then waits for a key to
 *        be pressed to stop them.
 *      - The top-left and bottom-left threads work in a loop, as follows: the
 *        top-left thread draws a rectangle in its buffer (in a size
 *        different from the previous rectangle), then sends an event to the 
 *        bottom-left thread. The bottom-left thread waits for the event from 
 *        the top-left thread, copies the contents of the top-left buffer into 
 *        its buffer, draws a circle in the rectangle, then sends an event to 
 *        the top-left thread. When the top-left thread receives the event, the 
 *        loop continues.
 *      - The top-right and bottom-right threads work exactly the same way as the
 *        top-left and bottom-left threads.
 *
 *      Note that the top and bottom threads (of each half) could be set to do
 *      something else while waiting for each other. 
 */

/* headers */
#include <stdlib.h> 
#include <stdio.h> 
#include <conio.h>
#include <process.h>
#include <windows.h>
#include <mil.h>

/* local defines */
#define IMAGE_FILE             M_IMAGE_PATH"bird.mim"
#define IMAGE_WIDTH            256
#define IMAGE_HEIGHT           240
#define DRAW_RADIUS_MAX        50
#define DRAW_CENTER_POSX       64
#define DRAW_CENTER_POSY       60
#define STRING_LENGHT_MAX      40
#define STRING_POS_X           10
#define STRING_POS_Y          220
#define STRING_TOP            "0"
#define STRING_BOTTOM         "0"

/* Function prototypes */
unsigned long MFTYPE TopThread(void *TParam);
unsigned long MFTYPE BotLeftThread(void *TParam);
unsigned long MFTYPE BotRightThread(void *TParam);
void BalanceThreadScheduling();


/* Thread paramaters structure */
typedef struct
   {
   MIL_ID SrcImageId;
   MIL_ID DstImageId;
   MIL_ID EventSendId;
   MIL_ID EventWaitId;
   MIL_ID EventEndId;
   MIL_ID EventEndBotId;
   long   *NumberOfIterPtr;
   long   *ComVarPtr;
   } THREAD_PARAM;

/* Main function: */
/* -------------- */
void main(void)
{ 
   MIL_ID MilApplication,        /* Application identifier.                   */
          MilSystem,             /* System identifier.                        */
          MilDisplay,            /* Display identifier.                       */
          MilImage,              /* Image buffer identifiers.                 */
          MilTopLeftImage,       /* Top left child image.                     */
          MilBotLeftImage,       /* Bottom left child image.                  */
          MilTopRightImage,      /* Top right child image.                    */
          MilBotRightImage,      /* Bottom right child image.                 */
          EventSendTopLeft,      /* Event send by top left thread.            */
          EventSendTopRight,     /* Event send by top right thread.           */
          EventWaitTopLeft,      /* Event waited on by top left thread.       */
          EventWaitTopRight,     /* Event waited on by top right thread.      */
          EventEndTopLeft,       /* Event used to exit top left thread.       */
          EventEndBotLeft,       /* Event used to exit bottom left thread.    */
          EventEndTopRight,      /* Event used to exit top right thread.      */
          EventEndBotRight;      /* Event used to exit bottom right thread.   */
   long   NumberOfTopLeft  = 0L, /* Number of top left threads iterations     */
          NumberOfBotLeft  = 0L, /* Number of bottom left threads iterations. */
          NumberOfTopRight = 0L, /* Number of top right threads iterations.   */
          NumberOfBotRight = 0L, /* Number of bottom right threads iterations.*/
          ComVarLeft  = 0L,      /* Communication variable for left thread.   */
          ComVarRight = 0L;      /* Communication variablefor right thread.   */
   THREAD_PARAM TParTopLeft,     /* Parameters passed to top left thread.     */
                TParBotLeft,     /* Parameters passed to bottom left thread.  */
                TParTopRight,    /* Parameters passed to top right thread.    */
                TParBotRight;    /* Parameters passed to bottom right thread. */
   HANDLE ThreadHandle[4];       /* Thread handles.                           */
   DWORD  ThreadId[4];           /* Thread Ids.                               */
 
   /* Allocate defaults. */
   MappAllocDefault(M_SETUP, &MilApplication, &MilSystem,
                             &MilDisplay, M_NULL, M_NULL);

   /* Allocate child buffers. */
   MbufAlloc2d(MilSystem, IMAGE_WIDTH*2, IMAGE_HEIGHT*2, 8, 
               M_IMAGE+M_PROC+M_DISP, &MilImage);
   MbufChild2d(MilImage, 0, 0, IMAGE_WIDTH  , IMAGE_HEIGHT,   &MilTopLeftImage);
   MbufChild2d(MilImage, IMAGE_WIDTH,  0, IMAGE_WIDTH, IMAGE_HEIGHT, &MilTopRightImage);
   MbufChild2d(MilImage, 0, IMAGE_HEIGHT, IMAGE_WIDTH, IMAGE_HEIGHT, &MilBotLeftImage);
   MbufChild2d(MilImage, IMAGE_WIDTH, IMAGE_HEIGHT, IMAGE_WIDTH, IMAGE_HEIGHT, 
               &MilBotRightImage);
   MbufClear(MilImage, 0);                                      
   MdispSelect(MilDisplay,MilImage);
 
   /* Allocate synchronization events. */
   MappControlThread(M_DEFAULT, M_EVENT_ALLOC, M_DEFAULT, &EventSendTopLeft);
   MappControlThread(M_DEFAULT, M_EVENT_ALLOC, M_DEFAULT, &EventSendTopRight);
   MappControlThread(M_DEFAULT, M_EVENT_ALLOC, M_DEFAULT, &EventWaitTopLeft);
   MappControlThread(M_DEFAULT, M_EVENT_ALLOC, M_DEFAULT, &EventWaitTopRight);
   MappControlThread(M_DEFAULT, M_EVENT_ALLOC, M_DEFAULT, &EventEndTopLeft);
   MappControlThread(M_DEFAULT, M_EVENT_ALLOC, M_DEFAULT, &EventEndTopRight);
   MappControlThread(M_DEFAULT, M_EVENT_ALLOC, M_DEFAULT, &EventEndBotLeft);
   MappControlThread(M_DEFAULT, M_EVENT_ALLOC, M_DEFAULT, &EventEndBotRight);
   
   /* Initialize source buffers. */
   MbufLoad(IMAGE_FILE,MilTopLeftImage);
   MbufLoad(IMAGE_FILE,MilTopRightImage);

   /* Intitalize threads parameter structures. */
   TParTopLeft.SrcImageId       = MilTopLeftImage;
   TParTopLeft.DstImageId       = MilTopLeftImage;
   TParTopLeft.EventSendId      = EventSendTopLeft;
   TParTopLeft.EventWaitId      = EventWaitTopLeft;
   TParTopLeft.EventEndId       = EventEndTopLeft;
   TParTopLeft.EventEndBotId    = EventEndBotLeft;
   TParTopLeft.NumberOfIterPtr  = &NumberOfTopLeft;
   TParTopLeft.ComVarPtr        = &ComVarLeft;

   TParBotLeft.SrcImageId       = MilTopLeftImage;
   TParBotLeft.DstImageId       = MilBotLeftImage;
   TParBotLeft.EventSendId      = EventWaitTopLeft;
   TParBotLeft.EventWaitId      = EventSendTopLeft;
   TParBotLeft.EventEndId       = EventEndBotLeft;
   TParBotLeft.EventEndBotId    = M_NULL;
   TParBotLeft.NumberOfIterPtr  = &NumberOfBotLeft;
   TParBotLeft.ComVarPtr        = &ComVarLeft;

   TParTopRight.SrcImageId      = MilTopRightImage;
   TParTopRight.DstImageId      = MilTopRightImage;
   TParTopRight.EventSendId     = EventSendTopRight;
   TParTopRight.EventWaitId     = EventWaitTopRight;
   TParTopRight.EventEndId      = EventEndTopRight;
   TParTopRight.EventEndBotId   = EventEndBotRight;
   TParTopRight.NumberOfIterPtr = &NumberOfTopRight;
   TParTopRight.ComVarPtr       = &ComVarRight;
   
   TParBotRight.SrcImageId      = MilTopRightImage;
   TParBotRight.DstImageId      = MilBotRightImage;
   TParBotRight.EventSendId     = EventWaitTopRight;
   TParBotRight.EventWaitId     = EventSendTopRight;
   TParBotRight.EventEndId      = EventEndBotRight;
   TParBotRight.EventEndBotId   = M_NULL;
   TParBotRight.NumberOfIterPtr = &NumberOfBotRight;
   TParBotRight.ComVarPtr       = &ComVarRight;
   
   /* Start drawing threads. */
   ThreadHandle[0] = (HANDLE) CreateThread(NULL, 0L, (LPTHREAD_START_ROUTINE) &TopThread      , &TParTopLeft,  0L, &(ThreadId[0]));
   ThreadHandle[1] = (HANDLE) CreateThread(NULL, 0L, (LPTHREAD_START_ROUTINE) &BotLeftThread  , &TParBotLeft,  0L, &(ThreadId[1]));
   ThreadHandle[2] = (HANDLE) CreateThread(NULL, 0L, (LPTHREAD_START_ROUTINE) &TopThread      , &TParTopRight, 0L, &(ThreadId[2]));
   ThreadHandle[3] = (HANDLE) CreateThread(NULL, 0L, (LPTHREAD_START_ROUTINE) &BotRightThread , &TParBotRight, 0L, &(ThreadId[3]));
 
   /* Send events to trigger operation of top left and top right threads. */
   MappControlThread(EventWaitTopLeft ,M_EVENT_SET, M_SIGNALED,M_NULL);
   MappControlThread(EventWaitTopRight,M_EVENT_SET, M_SIGNALED,M_NULL);

   /* Report what has happened to the Host screen. */ 
   printf("Drawing done in a loop using four threads.\n");
   printf("Press <Enter> to continue.\n");
   getchar();
   
   /* Make all threads exit. */
   MappControlThread(EventEndTopLeft , M_EVENT_SET, M_SIGNALED, M_NULL);
   MappControlThread(EventEndTopRight, M_EVENT_SET, M_SIGNALED, M_NULL);
   
   /* Wait before freeing MIL objects that all threads are finished. */
   while ((MappControlThread(EventEndTopLeft , M_EVENT_STATE, M_DEFAULT, M_NULL) == M_SIGNALED) ||
          (MappControlThread(EventEndTopRight, M_EVENT_STATE, M_DEFAULT, M_NULL) == M_SIGNALED)   )
         ; 
          
   printf("Top left iterations done:     %4ld.\n", NumberOfTopLeft);
   printf("Bottom left iterations done:  %4ld.\n", NumberOfBotLeft);
   printf("Top right iterations done:    %4ld.\n", NumberOfTopRight);
   printf("Bottom right iterations done: %4ld.\n", NumberOfBotRight);
   printf("Press <Enter> to end.\n");
   getchar();
  
   /* Free buffers. */
   MappControlThread(EventSendTopLeft , M_EVENT_FREE, M_DEFAULT, M_NULL);
   MappControlThread(EventSendTopRight, M_EVENT_FREE, M_DEFAULT, M_NULL);
   MappControlThread(EventWaitTopLeft , M_EVENT_FREE, M_DEFAULT, M_NULL);
   MappControlThread(EventWaitTopRight, M_EVENT_FREE, M_DEFAULT, M_NULL);
   MappControlThread(EventEndTopLeft  , M_EVENT_FREE, M_DEFAULT, M_NULL);
   MappControlThread(EventEndTopRight , M_EVENT_FREE, M_DEFAULT, M_NULL);
   MappControlThread(EventEndBotLeft  , M_EVENT_FREE, M_DEFAULT, M_NULL);
   MappControlThread(EventEndBotRight , M_EVENT_FREE, M_DEFAULT, M_NULL);
   MbufFree(MilTopLeftImage);
   MbufFree(MilTopRightImage);
   MbufFree(MilBotLeftImage);
   MbufFree(MilBotRightImage);
   MbufFree(MilImage);

   /* Release defaults. */
   MappFreeDefault(MilApplication, MilSystem, MilDisplay, M_NULL, M_NULL);
   }

/* Top left and top right functions: */
/* --------------------------------- */
unsigned long MFTYPE TopThread(void *TParam)
{
   MIL_ID SrcImageId    = ((THREAD_PARAM *) TParam)->SrcImageId;
   MIL_ID DstImageId    = ((THREAD_PARAM *) TParam)->DstImageId;
   MIL_ID EventSendId   = ((THREAD_PARAM *) TParam)->EventSendId;
   MIL_ID EventWaitId   = ((THREAD_PARAM *) TParam)->EventWaitId;
   MIL_ID EventEndId    = ((THREAD_PARAM *) TParam)->EventEndId;
   MIL_ID EventEndBotId = ((THREAD_PARAM *) TParam)->EventEndBotId;
   long * RadiusVarPtr  = ((THREAD_PARAM *) TParam)->ComVarPtr;
   char   Text[STRING_LENGHT_MAX] = STRING_TOP;
   long   Exit=0;
   
   while (!Exit)
      {
      /* Wait for event to process. */
      MappControlThread(EventWaitId, M_EVENT_WAIT, M_DEFAULT, M_NULL);
      
      /* Modify communication variable and reload image if necessary. */
      if (*RadiusVarPtr < DRAW_RADIUS_MAX)
         {
         *RadiusVarPtr += 10;   
         }
      else    
         {
         *RadiusVarPtr = 0;   
         MbufLoad(IMAGE_FILE, DstImageId);
         }
      
      /* Print number of iterations and draw. */
      (*(((THREAD_PARAM *) TParam)->NumberOfIterPtr))+= 1;   
      ltoa(*(((THREAD_PARAM *) TParam)->NumberOfIterPtr), Text, 10);
      MgraColor(M_DEFAULT, 0xff);
      MgraText(M_DEFAULT, SrcImageId, STRING_POS_X, STRING_POS_Y, Text);
      MgraRectFill(M_DEFAULT, DstImageId, DRAW_CENTER_POSX-*RadiusVarPtr,
                                          DRAW_CENTER_POSY-*RadiusVarPtr,
                                          DRAW_CENTER_POSX+*RadiusVarPtr,
                                          DRAW_CENTER_POSY+*RadiusVarPtr);
   
      /* Check if processing must be terminated. */
      if (MappControlThread(EventEndId, M_EVENT_STATE, M_DEFAULT, M_NULL) == M_SIGNALED)
         {
         /* Make bottom thread exit. */
         MappControlThread(EventEndBotId, M_EVENT_SET, M_SIGNALED, M_NULL);
         
         /* Set exit loop flag. */
         Exit=1;
         }

      /* Synchronize main thread with end of drawing. */
      MappControlThread(EventSendId, M_EVENT_SET, M_SIGNALED, M_NULL);
      }
   
   /* Minimize impact of Window's thread scheduling adjustment mechanisms. */
   BalanceThreadScheduling();

   /* Wait before freeing MIL objects that all threads are finished. */
   while (MappControlThread(EventEndBotId, M_EVENT_STATE, M_DEFAULT, M_NULL) == M_SIGNALED);
      
   /* Make sure that exit of thread is synchronized with HOST. */
   MappControlThread(EventEndId, M_EVENT_SET, M_NOT_SIGNALED, M_NULL);
   return(1L);
   }
   
/* Bottom left functions: */
/* ---------------------- */
unsigned long MFTYPE BotLeftThread(void *TParam)
{
   MIL_ID SrcImageId   = ((THREAD_PARAM *) TParam)->SrcImageId;
   MIL_ID DstImageId   = ((THREAD_PARAM *) TParam)->DstImageId;
   MIL_ID EventSendId  = ((THREAD_PARAM *) TParam)->EventSendId;
   MIL_ID EventWaitId  = ((THREAD_PARAM *) TParam)->EventWaitId;
   MIL_ID EventEndId   = ((THREAD_PARAM *) TParam)->EventEndId;
   long * RadiusVarPtr = ((THREAD_PARAM *) TParam)->ComVarPtr;
   char   Text[STRING_LENGHT_MAX] = STRING_BOTTOM;
   long   Exit=0;
   
   while (!Exit)
      {
      /* Wait for event to process. */
      MappControlThread(EventWaitId, M_EVENT_WAIT, M_DEFAULT, M_NULL);
      
      /* Increment number of iterations. */
      (*(((THREAD_PARAM *) TParam)->NumberOfIterPtr))+= 01L;

      /* Copy the top image. */
      MbufCopy(SrcImageId,DstImageId);

      /* Print iteration count and draw. */
      ltoa(*(((THREAD_PARAM *) TParam)->NumberOfIterPtr), Text, 10);
      MgraColor(M_DEFAULT, 0xFF);
      MgraText(M_DEFAULT, DstImageId, STRING_POS_X, STRING_POS_Y, Text);
      MgraColor(M_DEFAULT, 0x80);
      MgraArcFill(M_DEFAULT, DstImageId, DRAW_CENTER_POSX, DRAW_CENTER_POSY,
                  *RadiusVarPtr, *RadiusVarPtr, 0, 360);
   
      /* Check if processing must be terminated. */
      if (MappControlThread(EventEndId, M_EVENT_STATE, M_DEFAULT, M_NULL) == M_SIGNALED)
         {
         /* Set exit loop flag. */
         Exit=1;
         }

      /* Minimize impact of Window's thread scheduling adjustment mechanisms. */
      BalanceThreadScheduling();
      
      /* Synchronize main thread with end of drawing. */
      MappControlThread(EventSendId, M_EVENT_SET, M_SIGNALED, M_NULL);
      }
      
   /* Make that exit of thread is synchronized with HOST. */
   MappControlThread(EventEndId, M_EVENT_SET, M_NOT_SIGNALED, M_NULL);
   return(1L);
}

/* Bottom right function: */
/* ---------------------- */
unsigned long MFTYPE BotRightThread(void *TParam)
{
   MIL_ID SrcImageId   = ((THREAD_PARAM *) TParam)->SrcImageId;
   MIL_ID DstImageId   = ((THREAD_PARAM *) TParam)->DstImageId;
   MIL_ID EventSendId  = ((THREAD_PARAM *) TParam)->EventSendId;
   MIL_ID EventWaitId  = ((THREAD_PARAM *) TParam)->EventWaitId;
   MIL_ID EventEndId   = ((THREAD_PARAM *) TParam)->EventEndId;
   long * RadiusVarPtr = ((THREAD_PARAM *) TParam)->ComVarPtr;
   char   Text[STRING_LENGHT_MAX] = STRING_BOTTOM;
   long   Exit=0;
   
   while (!Exit)
      {
      /* Wait for event to process. */
      MappControlThread(EventWaitId, M_EVENT_WAIT, M_DEFAULT, M_NULL);
      
      /* Increment number of iterations. */
      (*(((THREAD_PARAM *) TParam)->NumberOfIterPtr))+= 01L;

      /* Copy the top image. */
      MbufCopy(SrcImageId,DstImageId);

      /* Print iteration count and draw. */
      ltoa(*(((THREAD_PARAM *) TParam)->NumberOfIterPtr), Text, 10);
      MgraColor(M_DEFAULT, 0xFF);
      MgraText(M_DEFAULT, DstImageId, STRING_POS_X, STRING_POS_Y, Text);
      MgraColor(M_DEFAULT, 0x40);
      MgraArcFill(M_DEFAULT, DstImageId, DRAW_CENTER_POSX, DRAW_CENTER_POSY,
                  *RadiusVarPtr/2, *RadiusVarPtr/2, 0, 360);
   
      /* Check if processing must be terminated. */
      if (MappControlThread(EventEndId, M_EVENT_STATE, M_DEFAULT, M_NULL) == M_SIGNALED)
         {
         /* Set exit loop flag. */
         Exit=1;
         }

      /* Minimize impact of Window's thread scheduling adjustment mechanisms. */
      BalanceThreadScheduling();
      
      /* Synchronize main thread with end of drawing. */
      MappControlThread(EventSendId, M_EVENT_SET, M_SIGNALED, M_NULL);
      }
      
   /* Make sure that exit of thread is synchronized with HOST. */
   MappControlThread(EventEndId, M_EVENT_SET, M_NOT_SIGNALED, M_NULL);
   return(1L);
}
   

/* Thread Scheduling Adjustment Handling */
/* ------------------------------------- */
/* NOTE: In some situations, Windows adjusts the priority or quantum value of threads.   */
/*       The intent of these adjustments is to improve system throughput and             */
/*       responsiveness. However, like any scheduling algorithms these adjustment are    */
/*       not perfect, and they might not be beneficial to all applications. The          */
/*       following function is introduced to minimize the impact of thread scheduling    */
/*       adjustment as performed by Windows. Refer to the book "Inside Windows NT        */
/*       (Second Edition, Microsoft Press, 1998, ISBN 1-57231-677-2)" for more details   */
/*       about thread scheduling under Windows.                                          */
void BalanceThreadScheduling()
{
   Sleep(0);
}
   
