/* File name: channel_switch.c 
 * Synopsis: This example shows how to adjust you camera by
 *           grabbing continuously on multiple channels.
 *
 * Note: This assumes that you use a Meteor-II/Standard with twelve
 *       channels. An application note on Fast Channel switching
 *       (channel_switch.pdf) is available on the Matrox Imaging website
 *       at http://www.matrox.com/imaging/
 */


#include <stdio.h>
#include <mil.h> 
#include <conio.h>

/* DCF name.
 * For grab field mode, use only DCF filenames finishing by "_fs.dcf".
 *                      Valid DCF are: rs170_fs.dcf, ntsc_fs.dcf, 
 *                                     ccir_fs.dcf and pal_fs.dcf
 * When grabbing whole frames, use standard DCF.
 */
#define DCF_NAME    "rs170_fs.DCF"

/* Image scaling. */
#define IMAGE_SCALE 0.5

/*Number of Channel */
#define NUM_CHANNELS 2

/* Buffer layout. */
#define NB_IMAGE_X  2
#define NB_IMAGE_Y  1


void main(void)
   { 
   MIL_ID MilApplication,                           /* Application identifier.   */
          MilSystem,                                /* System identifier.        */
          MilDisplay,                               /* Display identifier.       */ 
          MilDigitizer,                             /* Digitizer identifier.     */ 
          MilImageDisp,                             /* Image buffer identifier.  */
          MilImageDispChild[NB_IMAGE_X*NB_IMAGE_Y], /* Image buffer identifier.  */
          MilImageHost[NUM_CHANNELS],               /* Image buffer identifier.  */
          ColorLock;                                /* Image buffer identifier.  */
      
   long    Counter, 
           NbGrab = 0,
           X,
           Y;

   long    Channel[] = {M_CH0, M_CH1, M_CH2,  M_CH3, 
                        M_CH4, M_CH5, M_CH6,  M_CH7,
                        M_CH8, M_CH9, M_CH10, M_CH11};

   long    Color[NUM_CHANNELS];
   double  Wait = 0.1;
   double  TimeStart,
           TimeEnd;

   unsigned long  AGCValue [NUM_CHANNELS];
   
   
   /* Allocate all the parameters. */
   MappAlloc(M_DEFAULT, &MilApplication);
   MsysAlloc(M_SYSTEM_METEOR_II, M_DEF_SYSTEM_NUM, M_SETUP, &MilSystem);
   MdispAlloc(MilSystem, M_DEFAULT, M_DEF_DISPLAY_FORMAT, M_DEFAULT, &MilDisplay);
   MdigAlloc(MilSystem, M_DEFAULT, DCF_NAME, M_DEFAULT, &MilDigitizer);
   
   /* Allocate a display buffer, clear it and display it. */
   MbufAllocColor(MilSystem,
                  MdigInquire(MilDigitizer, M_SIZE_BAND, M_NULL),
                  (long) (MdigInquire(MilDigitizer, M_SIZE_X, M_NULL)*IMAGE_SCALE*NB_IMAGE_X),
                  (long) (MdigInquire(MilDigitizer, M_SIZE_Y, M_NULL)*IMAGE_SCALE*NB_IMAGE_Y),
                  8L+M_UNSIGNED, 
                  M_IMAGE+M_GRAB+M_DISP,
                  &MilImageDisp);
   
   MbufClear(MilImageDisp, 128L);
   MdispSelect(MilDisplay, MilImageDisp);
      
   /* Allocate one child buffer for each input Channel in the display buffer. */
   Counter = 0;
   for (Y = 0; Y < NB_IMAGE_Y; Y++)
      {
      for (X = 0; X < NB_IMAGE_X; X++)
         {
         MbufChild2d(MilImageDisp,
                     (long)(MdigInquire(MilDigitizer, M_SIZE_X, M_NULL)*IMAGE_SCALE*X),
                     (long)(MdigInquire(MilDigitizer, M_SIZE_Y, M_NULL)*IMAGE_SCALE*Y),
                     (long)(MdigInquire(MilDigitizer, M_SIZE_X, M_NULL)*IMAGE_SCALE),
                     (long)(MdigInquire(MilDigitizer, M_SIZE_Y, M_NULL)*IMAGE_SCALE),
                     &MilImageDispChild[Counter++]);
         }
      }   
   
   /* Allocate one host buffer for each input Channel. */
   for (Counter = 0; Counter < NUM_CHANNELS; Counter++)
      {
      MbufAllocColor(MilSystem,
                     MdigInquire(MilDigitizer, M_SIZE_BAND, M_NULL),
                     (long)(MdigInquire(MilDigitizer, M_SIZE_X, M_NULL)*IMAGE_SCALE),
                     (long)(MdigInquire(MilDigitizer, M_SIZE_Y, M_NULL)*IMAGE_SCALE),
                     8L+M_UNSIGNED, 
                     M_IMAGE+M_GRAB,
                     &MilImageHost[Counter]);
      }  
  
   /* Enable fast switching synchronisation */  
   MdigControl(MilDigitizer, M_SYNCHRONIZE_CHANNEL, M_ENABLE);

   /* Read gain setting for each channel */
   for (Counter = 0; Counter < NUM_CHANNELS; Counter++)
      {      
      MdigControl(MilDigitizer, M_GRAB_AUTOMATIC_INPUT_GAIN, M_ENABLE); 
      MdigChannel(MilDigitizer, Channel[Counter]);
      MappTimer(M_TIMER_WAIT,&Wait);
      ColorLock=M_FALSE;
      MdigInquire(MilDigitizer,M_INPUT_SIGNAL_COLOR_LOCK,&ColorLock);
      if(ColorLock == 0)
         {
         Color[Counter]=0;       /* camera is monochrome */
         MdigControl(MilDigitizer, M_GRAB_AUTOMATIC_INPUT_GAIN, M_DISABLE); 
         MdigInquire (MilDigitizer, M_GRAB_INPUT_GAIN, &AGCValue[Counter]);
         }
      else
         {
         Color[Counter]=1;       /* camera is Color */
         MdigControl(MilDigitizer, M_GRAB_AUTOMATIC_INPUT_GAIN, M_DISABLE); 
         MdigInquire (MilDigitizer, M_GRAB_INPUT_GAIN, &AGCValue[Counter]);
         }
      }
         
   /* Set the acquisition in synchronous mode. */
   MdigControl(MilDigitizer, M_GRAB_MODE, M_SYNCHRONOUS);   
   
   /* Set the acquisition to start on any field. */
   MdigControl(MilDigitizer, M_GRAB_START_MODE, M_FIELD_START);   
   
   /* Set the acquisition scaling. */
   MdigControl(MilDigitizer, M_GRAB_SCALE, IMAGE_SCALE);
   
   /* Set the acquisition to grab only one field. */
   MdigControl(MilDigitizer, M_GRAB_FIELD_NUM, 1);
   
   /* Disable automatic input gain */
   MdigControl(MilDigitizer, M_GRAB_AUTOMATIC_INPUT_GAIN, M_DISABLE);
   
   /* When a key is pressed, Start. */
   printf("Press <Enter> to start grabbing.\n");
   getchar();
   printf("Press <Enter> to stop grabbing and exit.\n");
   
   /* Grab the sequence. */
   /* To start on Channel 0 */
   Counter = NUM_CHANNELS -1;

   /* Start the timer */
   MappTimer(M_TIMER_READ, &TimeStart);

   while(!kbhit())
      {
      /* Select an input Channel. */
      if (Counter < NUM_CHANNELS - 1)
         Counter++;
      else
         Counter = 0;
      
      /* Set the correct gain for this Channel */
      MdigControl(MilDigitizer, M_GRAB_INPUT_GAIN, AGCValue[Counter]);
      
      /* Switch Channel */
      if (Color[Counter])
         MdigChannel(MilDigitizer, Channel[Counter]+M_CHANNEL_COLOR);
      else
         MdigChannel(MilDigitizer, Channel[Counter]);
      
      MdigGrab(MilDigitizer, MilImageDispChild[Counter]);
      NbGrab++;
      } /* End of while ... */
   
   /* Stop the timer */
   MappTimer(M_TIMER_READ, &TimeEnd);
   
   printf("Performance %f  en %d  en %fs\n", NbGrab/((TimeEnd-TimeStart)), NbGrab, (TimeEnd-TimeStart));
   printf("Press <Enter> to end.\n");    
   getchar();
      
   /* Free allocations. */
   for (Counter = 0; Counter < NUM_CHANNELS; Counter++)
      {
      MbufFree(MilImageHost[Counter]);
      }

   for (Counter = 0; Counter < NB_IMAGE_X*NB_IMAGE_Y; Counter++)
      {
      MbufFree(MilImageDispChild[Counter]);
      }

   MbufFree(MilImageDisp);
   MdigFree(MilDigitizer);
   MdispFree(MilDisplay);
   MsysFree(MilSystem);
   MappFree(MilApplication);   
}


