/********************************************************************
 *
 * Real-time processing and display with Genesis-LC.
 * Use this model if you want to display the original unprocessed
 * image, not the processed one.
 * (See LC1.C if you want to display the processed image).
 *
 * The example demonstrates how grab is double-buffered
 * to achieve real-time processing. Asynchronous grab mode is used
 * so that only a single thread is needed.
 *
 * Note that the grab destination is host memory, but a control field
 * specifies that a displayable buffer be used as the on-board work
 * buffer for the grab (i.e., the grab is really done into display memory,
 * and a synchronized DMA transfer sends the image to host memory too.
 *
 ********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "imapi.h"

#define MAXBAND 4

/* Prototypes */
void ErrHandler(void *Param);

void main(int argc, char **argv)
{
    long Device;               /* Genesis device */
    long Camera;               /* Camera */
    long Thread;               /* Thread to execute all functions */
    long GrabOSB[2];           /* OSBs used for synchronization */
    long InBuf[2];             /* Double-buffered input buffer */
    long DispBuf;              /* Display buffer (full screen)*/
    long DispChild;            /* Display buffer (grab size) */            
    long CtrlBuf;              /* Control buffer */
    long SizeX, SizeY;         /* Image size */
    long NumBands;             /* Number of bands in image */
    long KeyVal = 6;           /* Keying colour in single screen mode */
    long i, Band, Niter = 0;
    long Start, End, Dummy;
    unsigned char *HostAddr[MAXBAND][2];
    double fps;
    long dispMode;

/* Check arguments */
    if (argc > 1 && *argv[1] == '?')
    {
        printf("Usage: LC2 [-x] [-y] [-k]\n");
        printf("       -x size\t Image X size\n");
        printf("       -y size\t Image Y size\n");
        printf("       -k val \t Keying colour (default %i)\n", KeyVal);
        exit(1);
    }

/* Establish an error handler */
    imAppCatchError(IM_DEFAULT, ErrHandler, NULL);

/* Allocate the device and a thread */
    imDevAlloc(0, 0, NULL, IM_DEFAULT, &Device);
    imThrAlloc(Device, 0, &Thread);

/* Allocate a camera */
    imCamAlloc(Thread, NULL, IM_DEFAULT, &Camera);

/* Determine the image size and number of bands */
    imCamInquire(Thread, Camera, IM_DIG_SIZE_X, &SizeX);
    imCamInquire(Thread, Camera, IM_DIG_SIZE_Y, &SizeY);
    imCamInquire(Thread, Camera, IM_DIG_NUM_BANDS, &NumBands);

/* Check if the user specified a different size or weight */
    for (i = 1; i < argc; i++)
    {
        if (!strcmp(argv[i], "-k"))
            sscanf(argv[i+1], "%i", &KeyVal);
        else if (!strcmp(argv[i], "-x"))
            sscanf(argv[i+1], "%li", &SizeX);
        else if (!strcmp(argv[i], "-y"))
            sscanf(argv[i+1], "%li", &SizeY);
    }

    printf("Image size: %ix%i\n", SizeX, SizeY);

/* Disable buffering of stdout */
    setvbuf(stdout, NULL, _IONBF, 0);

/* Allocate grab buffers in host memory */
    imBufAlloc(Thread, SizeX, SizeY, NumBands, IM_UBYTE, IM_HOST, &InBuf[0]);
    imBufAlloc(Thread, SizeX, SizeY, NumBands, IM_UBYTE, IM_HOST, &InBuf[1]);

/* Allocate control buffer */
    imBufAllocControl(Thread, &CtrlBuf);

/* Find host address of grab buffers (needed for host processing) */
    for (Band = 0; Band < NumBands; Band++)
    {
        imBufMap(Thread, InBuf[0], Band, 0, (void **)&HostAddr[Band][0], &Dummy, &Dummy);
        imBufMap(Thread, InBuf[1], Band, 0, (void **)&HostAddr[Band][1], &Dummy, &Dummy);
    }

/* Allocate a full-screen display buffer and clear it */
    imBufChild(Thread, IM_DISP, 0, 0, IM_ALL, IM_ALL, &DispBuf);
    imBufClear(Thread, DispBuf, 0, 0);

/* Allocate display child buffer at the required location */
    imBufChild(Thread, IM_DISP, 0, 0, SizeX, SizeY, &DispChild);

/* Enable keying if necessary */
    if ((imDispInquire(Thread, 0, IM_DISP_VGA_MODE, &dispMode) == IM_SINGLE_SCREEN) || (dispMode == IM_DUAL_HEAD))
    {
        if (imDispInquire(Thread, 0, IM_DISP_KEY_MODE, NULL) == IM_KEY_OFF)
        {
            imBufPutField(Thread, DispBuf, IM_DISP_KEY_MODE, IM_KEY_IN_RANGE);
            imBufPutField(Thread, DispBuf, IM_DISP_KEY_LOW, KeyVal);
            imBufPutField(Thread, DispBuf, IM_DISP_KEY_HIGH, KeyVal);
            imDispControl(Thread, 0, DispBuf, IM_FRAME);
        }
    }

/* Allocate OSBs for synchronization */
    imSyncAlloc(Thread, &GrabOSB[0]);
    imSyncAlloc(Thread, &GrabOSB[1]);

/* Select asynchronous grab mode */
    imBufPutField(Thread, CtrlBuf, IM_CTL_GRAB_MODE, IM_ASYNCHRONOUS);

/* Specfy that grab should use the display buffer */
    imBufPutField(Thread, CtrlBuf, IM_CTL_WORK_BUF, DispChild);

    printf("Press Enter to stop\n");

/* Make sure previous commands have finished before starting the timer */
    imSyncHost(Thread, 0, IM_COMPLETED);

/* Start the timer */
    Start = clock();

/* First grab */
    imDigGrab(Thread, 0, Camera, InBuf[0], 1, CtrlBuf, GrabOSB[0]);

    i = 1;
    while (!imAppGetError(IM_ERR_CODE, NULL))
    {
    /* Queue next grab into other buffer */
        imDigGrab(Thread, 0, Camera, InBuf[i], 1, CtrlBuf, GrabOSB[i]);
     
    /* Switch buffers */
        i = 1 - i;

    /* Process each frame as soon as the grab completes */
        imSyncHost(Thread, GrabOSB[i], IM_COMPLETED);

    /*
     * Insert host processing here (using host addresses already obtained).
     */

    /* Stop when a key is hit */
        if (kbhit())
            break;

    /* Print the frame rate */
        Niter++;
        End = clock();
        if (End - Start > CLOCKS_PER_SEC)
        {
            fps = Niter * CLOCKS_PER_SEC / (double) (End - Start);
            printf("\r");
            printf("Frames/sec = %.1f  ", fps);
            Start = clock();
            Niter = 0;
        }
    }

/* Clean up */
    imSyncHost(Thread, GrabOSB[1-i], IM_COMPLETED);
    imBufFree(Thread, DispChild);
    imBufFree(Thread, DispBuf);
    imBufFree(Thread, InBuf[0]);
    imBufFree(Thread, InBuf[1]);
    imBufFree(Thread, CtrlBuf);
    imSyncFree(Thread, GrabOSB[0]);
    imSyncFree(Thread, GrabOSB[1]);
    imCamFree(Thread, Camera);
    imThrFree(Thread);
    imDevFree(Device);
}


void ErrHandler(void *Param)
{
    char Error[IM_ERR_SIZE];
    /* To remove compiler warning: no reference to parameter */
    Param = Param;
/*
 * Get the error message and print it. Don't reset the error
 * because we only want the first to be printed.
 */
    imAppGetError(IM_ERR_MSG_FUNC, Error);
    printf("%s\n", Error);
}

